# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class AccountEntitlement(Model):
    """AccountEntitlement.

    :param account_id: Gets or sets the id of the account to which the license belongs
    :type account_id: str
    :param assignment_date: Gets or sets the date the license was assigned
    :type assignment_date: datetime
    :param assignment_source: Assignment Source
    :type assignment_source: object
    :param date_created: Gets or sets the creation date of the user in this account
    :type date_created: datetime
    :param last_accessed_date: Gets or sets the date of the user last sign-in to this account
    :type last_accessed_date: datetime
    :param license:
    :type license: :class:`License <azure.devops.v5_1.licensing.models.License>`
    :param origin: Licensing origin
    :type origin: object
    :param rights: The computed rights of this user in the account.
    :type rights: :class:`AccountRights <azure.devops.v5_1.licensing.models.AccountRights>`
    :param status: The status of the user in the account
    :type status: object
    :param user: Identity information of the user to which the license belongs
    :type user: :class:`IdentityRef <azure.devops.v5_1.licensing.models.IdentityRef>`
    :param user_id: Gets the id of the user to which the license belongs
    :type user_id: str
    """

    _attribute_map = {
        'account_id': {'key': 'accountId', 'type': 'str'},
        'assignment_date': {'key': 'assignmentDate', 'type': 'iso-8601'},
        'assignment_source': {'key': 'assignmentSource', 'type': 'object'},
        'date_created': {'key': 'dateCreated', 'type': 'iso-8601'},
        'last_accessed_date': {'key': 'lastAccessedDate', 'type': 'iso-8601'},
        'license': {'key': 'license', 'type': 'License'},
        'origin': {'key': 'origin', 'type': 'object'},
        'rights': {'key': 'rights', 'type': 'AccountRights'},
        'status': {'key': 'status', 'type': 'object'},
        'user': {'key': 'user', 'type': 'IdentityRef'},
        'user_id': {'key': 'userId', 'type': 'str'}
    }

    def __init__(self, account_id=None, assignment_date=None, assignment_source=None, date_created=None, last_accessed_date=None, license=None, origin=None, rights=None, status=None, user=None, user_id=None):
        super(AccountEntitlement, self).__init__()
        self.account_id = account_id
        self.assignment_date = assignment_date
        self.assignment_source = assignment_source
        self.date_created = date_created
        self.last_accessed_date = last_accessed_date
        self.license = license
        self.origin = origin
        self.rights = rights
        self.status = status
        self.user = user
        self.user_id = user_id


class AccountEntitlementUpdateModel(Model):
    """AccountEntitlementUpdateModel.

    :param license: Gets or sets the license for the entitlement
    :type license: :class:`License <azure.devops.v5_1.licensing.models.License>`
    """

    _attribute_map = {
        'license': {'key': 'license', 'type': 'License'}
    }

    def __init__(self, license=None):
        super(AccountEntitlementUpdateModel, self).__init__()
        self.license = license


class AccountLicenseExtensionUsage(Model):
    """AccountLicenseExtensionUsage.

    :param extension_id:
    :type extension_id: str
    :param extension_name:
    :type extension_name: str
    :param included_quantity:
    :type included_quantity: int
    :param is_trial:
    :type is_trial: bool
    :param minimum_license_required:
    :type minimum_license_required: object
    :param msdn_used_count:
    :type msdn_used_count: int
    :param provisioned_count:
    :type provisioned_count: int
    :param remaining_trial_days:
    :type remaining_trial_days: int
    :param trial_expiry_date:
    :type trial_expiry_date: datetime
    :param used_count:
    :type used_count: int
    """

    _attribute_map = {
        'extension_id': {'key': 'extensionId', 'type': 'str'},
        'extension_name': {'key': 'extensionName', 'type': 'str'},
        'included_quantity': {'key': 'includedQuantity', 'type': 'int'},
        'is_trial': {'key': 'isTrial', 'type': 'bool'},
        'minimum_license_required': {'key': 'minimumLicenseRequired', 'type': 'object'},
        'msdn_used_count': {'key': 'msdnUsedCount', 'type': 'int'},
        'provisioned_count': {'key': 'provisionedCount', 'type': 'int'},
        'remaining_trial_days': {'key': 'remainingTrialDays', 'type': 'int'},
        'trial_expiry_date': {'key': 'trialExpiryDate', 'type': 'iso-8601'},
        'used_count': {'key': 'usedCount', 'type': 'int'}
    }

    def __init__(self, extension_id=None, extension_name=None, included_quantity=None, is_trial=None, minimum_license_required=None, msdn_used_count=None, provisioned_count=None, remaining_trial_days=None, trial_expiry_date=None, used_count=None):
        super(AccountLicenseExtensionUsage, self).__init__()
        self.extension_id = extension_id
        self.extension_name = extension_name
        self.included_quantity = included_quantity
        self.is_trial = is_trial
        self.minimum_license_required = minimum_license_required
        self.msdn_used_count = msdn_used_count
        self.provisioned_count = provisioned_count
        self.remaining_trial_days = remaining_trial_days
        self.trial_expiry_date = trial_expiry_date
        self.used_count = used_count


class AccountLicenseUsage(Model):
    """AccountLicenseUsage.

    :param disabled_count: Amount that is disabled (Usually from licenses that were provisioned, but became invalid due to loss of subscription in a new billing cycle)
    :type disabled_count: int
    :param license:
    :type license: :class:`AccountUserLicense <azure.devops.v5_1.licensing.models.AccountUserLicense>`
    :param pending_provisioned_count: Amount that will be purchased in the next billing cycle
    :type pending_provisioned_count: int
    :param provisioned_count: Amount that has been purchased
    :type provisioned_count: int
    :param used_count: Amount currently being used.
    :type used_count: int
    """

    _attribute_map = {
        'disabled_count': {'key': 'disabledCount', 'type': 'int'},
        'license': {'key': 'license', 'type': 'AccountUserLicense'},
        'pending_provisioned_count': {'key': 'pendingProvisionedCount', 'type': 'int'},
        'provisioned_count': {'key': 'provisionedCount', 'type': 'int'},
        'used_count': {'key': 'usedCount', 'type': 'int'}
    }

    def __init__(self, disabled_count=None, license=None, pending_provisioned_count=None, provisioned_count=None, used_count=None):
        super(AccountLicenseUsage, self).__init__()
        self.disabled_count = disabled_count
        self.license = license
        self.pending_provisioned_count = pending_provisioned_count
        self.provisioned_count = provisioned_count
        self.used_count = used_count


class AccountRights(Model):
    """AccountRights.

    :param level:
    :type level: object
    :param reason:
    :type reason: str
    """

    _attribute_map = {
        'level': {'key': 'level', 'type': 'object'},
        'reason': {'key': 'reason', 'type': 'str'}
    }

    def __init__(self, level=None, reason=None):
        super(AccountRights, self).__init__()
        self.level = level
        self.reason = reason


class AccountUserLicense(Model):
    """AccountUserLicense.

    :param license:
    :type license: int
    :param source:
    :type source: object
    """

    _attribute_map = {
        'license': {'key': 'license', 'type': 'int'},
        'source': {'key': 'source', 'type': 'object'}
    }

    def __init__(self, license=None, source=None):
        super(AccountUserLicense, self).__init__()
        self.license = license
        self.source = source


class ClientRightsContainer(Model):
    """ClientRightsContainer.

    :param certificate_bytes:
    :type certificate_bytes: str
    :param token:
    :type token: str
    """

    _attribute_map = {
        'certificate_bytes': {'key': 'certificateBytes', 'type': 'str'},
        'token': {'key': 'token', 'type': 'str'}
    }

    def __init__(self, certificate_bytes=None, token=None):
        super(ClientRightsContainer, self).__init__()
        self.certificate_bytes = certificate_bytes
        self.token = token


class ExtensionAssignment(Model):
    """ExtensionAssignment.

    :param extension_gallery_id: Gets or sets the extension ID to assign.
    :type extension_gallery_id: str
    :param is_auto_assignment: Set to true if this a auto assignment scenario.
    :type is_auto_assignment: bool
    :param licensing_source: Gets or sets the licensing source.
    :type licensing_source: object
    :param user_ids: Gets or sets the user IDs to assign the extension to.
    :type user_ids: list of str
    """

    _attribute_map = {
        'extension_gallery_id': {'key': 'extensionGalleryId', 'type': 'str'},
        'is_auto_assignment': {'key': 'isAutoAssignment', 'type': 'bool'},
        'licensing_source': {'key': 'licensingSource', 'type': 'object'},
        'user_ids': {'key': 'userIds', 'type': '[str]'}
    }

    def __init__(self, extension_gallery_id=None, is_auto_assignment=None, licensing_source=None, user_ids=None):
        super(ExtensionAssignment, self).__init__()
        self.extension_gallery_id = extension_gallery_id
        self.is_auto_assignment = is_auto_assignment
        self.licensing_source = licensing_source
        self.user_ids = user_ids


class ExtensionAssignmentDetails(Model):
    """ExtensionAssignmentDetails.

    :param assignment_status:
    :type assignment_status: object
    :param source_collection_name:
    :type source_collection_name: str
    """

    _attribute_map = {
        'assignment_status': {'key': 'assignmentStatus', 'type': 'object'},
        'source_collection_name': {'key': 'sourceCollectionName', 'type': 'str'}
    }

    def __init__(self, assignment_status=None, source_collection_name=None):
        super(ExtensionAssignmentDetails, self).__init__()
        self.assignment_status = assignment_status
        self.source_collection_name = source_collection_name


class ExtensionLicenseData(Model):
    """ExtensionLicenseData.

    :param created_date:
    :type created_date: datetime
    :param extension_id:
    :type extension_id: str
    :param is_free:
    :type is_free: bool
    :param minimum_required_access_level:
    :type minimum_required_access_level: object
    :param updated_date:
    :type updated_date: datetime
    """

    _attribute_map = {
        'created_date': {'key': 'createdDate', 'type': 'iso-8601'},
        'extension_id': {'key': 'extensionId', 'type': 'str'},
        'is_free': {'key': 'isFree', 'type': 'bool'},
        'minimum_required_access_level': {'key': 'minimumRequiredAccessLevel', 'type': 'object'},
        'updated_date': {'key': 'updatedDate', 'type': 'iso-8601'}
    }

    def __init__(self, created_date=None, extension_id=None, is_free=None, minimum_required_access_level=None, updated_date=None):
        super(ExtensionLicenseData, self).__init__()
        self.created_date = created_date
        self.extension_id = extension_id
        self.is_free = is_free
        self.minimum_required_access_level = minimum_required_access_level
        self.updated_date = updated_date


class ExtensionOperationResult(Model):
    """ExtensionOperationResult.

    :param account_id:
    :type account_id: str
    :param extension_id:
    :type extension_id: str
    :param message:
    :type message: str
    :param operation:
    :type operation: object
    :param result:
    :type result: object
    :param user_id:
    :type user_id: str
    """

    _attribute_map = {
        'account_id': {'key': 'accountId', 'type': 'str'},
        'extension_id': {'key': 'extensionId', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'operation': {'key': 'operation', 'type': 'object'},
        'result': {'key': 'result', 'type': 'object'},
        'user_id': {'key': 'userId', 'type': 'str'}
    }

    def __init__(self, account_id=None, extension_id=None, message=None, operation=None, result=None, user_id=None):
        super(ExtensionOperationResult, self).__init__()
        self.account_id = account_id
        self.extension_id = extension_id
        self.message = message
        self.operation = operation
        self.result = result
        self.user_id = user_id


class ExtensionRightsResult(Model):
    """ExtensionRightsResult.

    :param entitled_extensions:
    :type entitled_extensions: list of str
    :param host_id:
    :type host_id: str
    :param reason:
    :type reason: str
    :param reason_code:
    :type reason_code: object
    :param result_code:
    :type result_code: object
    """

    _attribute_map = {
        'entitled_extensions': {'key': 'entitledExtensions', 'type': '[str]'},
        'host_id': {'key': 'hostId', 'type': 'str'},
        'reason': {'key': 'reason', 'type': 'str'},
        'reason_code': {'key': 'reasonCode', 'type': 'object'},
        'result_code': {'key': 'resultCode', 'type': 'object'}
    }

    def __init__(self, entitled_extensions=None, host_id=None, reason=None, reason_code=None, result_code=None):
        super(ExtensionRightsResult, self).__init__()
        self.entitled_extensions = entitled_extensions
        self.host_id = host_id
        self.reason = reason
        self.reason_code = reason_code
        self.result_code = result_code


class ExtensionSource(Model):
    """ExtensionSource.

    :param assignment_source: Assignment Source
    :type assignment_source: object
    :param extension_gallery_id: extension Identifier
    :type extension_gallery_id: str
    :param licensing_source: The licensing source of the extension. Account, Msdn, ect.
    :type licensing_source: object
    """

    _attribute_map = {
        'assignment_source': {'key': 'assignmentSource', 'type': 'object'},
        'extension_gallery_id': {'key': 'extensionGalleryId', 'type': 'str'},
        'licensing_source': {'key': 'licensingSource', 'type': 'object'}
    }

    def __init__(self, assignment_source=None, extension_gallery_id=None, licensing_source=None):
        super(ExtensionSource, self).__init__()
        self.assignment_source = assignment_source
        self.extension_gallery_id = extension_gallery_id
        self.licensing_source = licensing_source


class GraphSubjectBase(Model):
    """GraphSubjectBase.

    :param _links: This field contains zero or more interesting links about the graph subject. These links may be invoked to obtain additional relationships or more detailed information about this graph subject.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.ReferenceLinks>`
    :param descriptor: The descriptor is the primary way to reference the graph subject while the system is running. This field will uniquely identify the same graph subject across both Accounts and Organizations.
    :type descriptor: str
    :param display_name: This is the non-unique display name of the graph subject. To change this field, you must alter its value in the source provider.
    :type display_name: str
    :param url: This url is the full route to the source resource of this graph subject.
    :type url: str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, _links=None, descriptor=None, display_name=None, url=None):
        super(GraphSubjectBase, self).__init__()
        self._links = _links
        self.descriptor = descriptor
        self.display_name = display_name
        self.url = url


class IdentityBase(Model):
    """IdentityBase.

    :param custom_display_name: The custom display name for the identity (if any). Setting this property to an empty string will clear the existing custom display name. Setting this property to null will not affect the existing persisted value (since null values do not get sent over the wire or to the database)
    :type custom_display_name: str
    :param descriptor:
    :type descriptor: :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param id:
    :type id: str
    :param is_active:
    :type is_active: bool
    :param is_container:
    :type is_container: bool
    :param master_id:
    :type master_id: str
    :param member_ids:
    :type member_ids: list of str
    :param member_of:
    :type member_of: list of :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param members:
    :type members: list of :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param meta_type_id:
    :type meta_type_id: int
    :param properties:
    :type properties: :class:`object <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.object>`
    :param provider_display_name: The display name for the identity as specified by the source identity provider.
    :type provider_display_name: str
    :param resource_version:
    :type resource_version: int
    :param subject_descriptor:
    :type subject_descriptor: :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param unique_user_id:
    :type unique_user_id: int
    """

    _attribute_map = {
        'custom_display_name': {'key': 'customDisplayName', 'type': 'str'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'master_id': {'key': 'masterId', 'type': 'str'},
        'member_ids': {'key': 'memberIds', 'type': '[str]'},
        'member_of': {'key': 'memberOf', 'type': '[str]'},
        'members': {'key': 'members', 'type': '[str]'},
        'meta_type_id': {'key': 'metaTypeId', 'type': 'int'},
        'properties': {'key': 'properties', 'type': 'object'},
        'provider_display_name': {'key': 'providerDisplayName', 'type': 'str'},
        'resource_version': {'key': 'resourceVersion', 'type': 'int'},
        'subject_descriptor': {'key': 'subjectDescriptor', 'type': 'str'},
        'unique_user_id': {'key': 'uniqueUserId', 'type': 'int'}
    }

    def __init__(self, custom_display_name=None, descriptor=None, id=None, is_active=None, is_container=None, master_id=None, member_ids=None, member_of=None, members=None, meta_type_id=None, properties=None, provider_display_name=None, resource_version=None, subject_descriptor=None, unique_user_id=None):
        super(IdentityBase, self).__init__()
        self.custom_display_name = custom_display_name
        self.descriptor = descriptor
        self.id = id
        self.is_active = is_active
        self.is_container = is_container
        self.master_id = master_id
        self.member_ids = member_ids
        self.member_of = member_of
        self.members = members
        self.meta_type_id = meta_type_id
        self.properties = properties
        self.provider_display_name = provider_display_name
        self.resource_version = resource_version
        self.subject_descriptor = subject_descriptor
        self.unique_user_id = unique_user_id


class IdentityMapping(Model):
    """IdentityMapping.

    :param source_identity:
    :type source_identity: :class:`Identity <azure.devops.v5_1.licensing.models.Identity>`
    :param target_identity:
    :type target_identity: :class:`Identity <azure.devops.v5_1.licensing.models.Identity>`
    """

    _attribute_map = {
        'source_identity': {'key': 'sourceIdentity', 'type': 'Identity'},
        'target_identity': {'key': 'targetIdentity', 'type': 'Identity'}
    }

    def __init__(self, source_identity=None, target_identity=None):
        super(IdentityMapping, self).__init__()
        self.source_identity = source_identity
        self.target_identity = target_identity


class IdentityRef(GraphSubjectBase):
    """IdentityRef.

    :param _links: This field contains zero or more interesting links about the graph subject. These links may be invoked to obtain additional relationships or more detailed information about this graph subject.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.ReferenceLinks>`
    :param descriptor: The descriptor is the primary way to reference the graph subject while the system is running. This field will uniquely identify the same graph subject across both Accounts and Organizations.
    :type descriptor: str
    :param display_name: This is the non-unique display name of the graph subject. To change this field, you must alter its value in the source provider.
    :type display_name: str
    :param url: This url is the full route to the source resource of this graph subject.
    :type url: str
    :param directory_alias: Deprecated - Can be retrieved by querying the Graph user referenced in the "self" entry of the IdentityRef "_links" dictionary
    :type directory_alias: str
    :param id:
    :type id: str
    :param image_url: Deprecated - Available in the "avatar" entry of the IdentityRef "_links" dictionary
    :type image_url: str
    :param inactive: Deprecated - Can be retrieved by querying the Graph membership state referenced in the "membershipState" entry of the GraphUser "_links" dictionary
    :type inactive: bool
    :param is_aad_identity: Deprecated - Can be inferred from the subject type of the descriptor (Descriptor.IsAadUserType/Descriptor.IsAadGroupType)
    :type is_aad_identity: bool
    :param is_container: Deprecated - Can be inferred from the subject type of the descriptor (Descriptor.IsGroupType)
    :type is_container: bool
    :param is_deleted_in_origin:
    :type is_deleted_in_origin: bool
    :param profile_url: Deprecated - not in use in most preexisting implementations of ToIdentityRef
    :type profile_url: str
    :param unique_name: Deprecated - use Domain+PrincipalName instead
    :type unique_name: str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'directory_alias': {'key': 'directoryAlias', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'image_url': {'key': 'imageUrl', 'type': 'str'},
        'inactive': {'key': 'inactive', 'type': 'bool'},
        'is_aad_identity': {'key': 'isAadIdentity', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'is_deleted_in_origin': {'key': 'isDeletedInOrigin', 'type': 'bool'},
        'profile_url': {'key': 'profileUrl', 'type': 'str'},
        'unique_name': {'key': 'uniqueName', 'type': 'str'}
    }

    def __init__(self, _links=None, descriptor=None, display_name=None, url=None, directory_alias=None, id=None, image_url=None, inactive=None, is_aad_identity=None, is_container=None, is_deleted_in_origin=None, profile_url=None, unique_name=None):
        super(IdentityRef, self).__init__(_links=_links, descriptor=descriptor, display_name=display_name, url=url)
        self.directory_alias = directory_alias
        self.id = id
        self.image_url = image_url
        self.inactive = inactive
        self.is_aad_identity = is_aad_identity
        self.is_container = is_container
        self.is_deleted_in_origin = is_deleted_in_origin
        self.profile_url = profile_url
        self.unique_name = unique_name


class License(Model):
    """License.

    :param source: Gets the source of the license
    :type source: object
    """

    _attribute_map = {
        'source': {'key': 'source', 'type': 'object'}
    }

    def __init__(self, source=None):
        super(License, self).__init__()
        self.source = source


class MsdnEntitlement(Model):
    """MsdnEntitlement.

    :param entitlement_code: Entilement id assigned to Entitlement in Benefits Database.
    :type entitlement_code: str
    :param entitlement_name: Entitlement Name e.g. Downloads, Chat.
    :type entitlement_name: str
    :param entitlement_type: Type of Entitlement e.g. Downloads, Chat.
    :type entitlement_type: str
    :param is_activated: Entitlement activation status
    :type is_activated: bool
    :param is_entitlement_available: Entitlement availability
    :type is_entitlement_available: bool
    :param subscription_channel: Write MSDN Channel into CRCT (Retail,MPN,VL,BizSpark,DreamSpark,MCT,FTE,Technet,WebsiteSpark,Other)
    :type subscription_channel: str
    :param subscription_expiration_date: Subscription Expiration Date.
    :type subscription_expiration_date: datetime
    :param subscription_id: Subscription id which identifies the subscription itself. This is the Benefit Detail Guid from BMS.
    :type subscription_id: str
    :param subscription_level_code: Identifier of the subscription or benefit level.
    :type subscription_level_code: str
    :param subscription_level_name: Name of subscription level.
    :type subscription_level_name: str
    :param subscription_status: Subscription Status Code (ACT, PND, INA ...).
    :type subscription_status: str
    """

    _attribute_map = {
        'entitlement_code': {'key': 'entitlementCode', 'type': 'str'},
        'entitlement_name': {'key': 'entitlementName', 'type': 'str'},
        'entitlement_type': {'key': 'entitlementType', 'type': 'str'},
        'is_activated': {'key': 'isActivated', 'type': 'bool'},
        'is_entitlement_available': {'key': 'isEntitlementAvailable', 'type': 'bool'},
        'subscription_channel': {'key': 'subscriptionChannel', 'type': 'str'},
        'subscription_expiration_date': {'key': 'subscriptionExpirationDate', 'type': 'iso-8601'},
        'subscription_id': {'key': 'subscriptionId', 'type': 'str'},
        'subscription_level_code': {'key': 'subscriptionLevelCode', 'type': 'str'},
        'subscription_level_name': {'key': 'subscriptionLevelName', 'type': 'str'},
        'subscription_status': {'key': 'subscriptionStatus', 'type': 'str'}
    }

    def __init__(self, entitlement_code=None, entitlement_name=None, entitlement_type=None, is_activated=None, is_entitlement_available=None, subscription_channel=None, subscription_expiration_date=None, subscription_id=None, subscription_level_code=None, subscription_level_name=None, subscription_status=None):
        super(MsdnEntitlement, self).__init__()
        self.entitlement_code = entitlement_code
        self.entitlement_name = entitlement_name
        self.entitlement_type = entitlement_type
        self.is_activated = is_activated
        self.is_entitlement_available = is_entitlement_available
        self.subscription_channel = subscription_channel
        self.subscription_expiration_date = subscription_expiration_date
        self.subscription_id = subscription_id
        self.subscription_level_code = subscription_level_code
        self.subscription_level_name = subscription_level_name
        self.subscription_status = subscription_status


class ReferenceLinks(Model):
    """ReferenceLinks.

    :param links: The readonly view of the links.  Because Reference links are readonly, we only want to expose them as read only.
    :type links: dict
    """

    _attribute_map = {
        'links': {'key': 'links', 'type': '{object}'}
    }

    def __init__(self, links=None):
        super(ReferenceLinks, self).__init__()
        self.links = links


class Identity(IdentityBase):
    """Identity.

    :param custom_display_name: The custom display name for the identity (if any). Setting this property to an empty string will clear the existing custom display name. Setting this property to null will not affect the existing persisted value (since null values do not get sent over the wire or to the database)
    :type custom_display_name: str
    :param descriptor:
    :type descriptor: :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param id:
    :type id: str
    :param is_active:
    :type is_active: bool
    :param is_container:
    :type is_container: bool
    :param master_id:
    :type master_id: str
    :param member_ids:
    :type member_ids: list of str
    :param member_of:
    :type member_of: list of :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param members:
    :type members: list of :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param meta_type_id:
    :type meta_type_id: int
    :param properties:
    :type properties: :class:`object <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.object>`
    :param provider_display_name: The display name for the identity as specified by the source identity provider.
    :type provider_display_name: str
    :param resource_version:
    :type resource_version: int
    :param subject_descriptor:
    :type subject_descriptor: :class:`str <azure.devops.v5_1.microsoft._visual_studio._services._web_api.models.str>`
    :param unique_user_id:
    :type unique_user_id: int
    """

    _attribute_map = {
        'custom_display_name': {'key': 'customDisplayName', 'type': 'str'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'master_id': {'key': 'masterId', 'type': 'str'},
        'member_ids': {'key': 'memberIds', 'type': '[str]'},
        'member_of': {'key': 'memberOf', 'type': '[str]'},
        'members': {'key': 'members', 'type': '[str]'},
        'meta_type_id': {'key': 'metaTypeId', 'type': 'int'},
        'properties': {'key': 'properties', 'type': 'object'},
        'provider_display_name': {'key': 'providerDisplayName', 'type': 'str'},
        'resource_version': {'key': 'resourceVersion', 'type': 'int'},
        'subject_descriptor': {'key': 'subjectDescriptor', 'type': 'str'},
        'unique_user_id': {'key': 'uniqueUserId', 'type': 'int'},
    }

    def __init__(self, custom_display_name=None, descriptor=None, id=None, is_active=None, is_container=None, master_id=None, member_ids=None, member_of=None, members=None, meta_type_id=None, properties=None, provider_display_name=None, resource_version=None, subject_descriptor=None, unique_user_id=None):
        super(Identity, self).__init__(custom_display_name=custom_display_name, descriptor=descriptor, id=id, is_active=is_active, is_container=is_container, master_id=master_id, member_ids=member_ids, member_of=member_of, members=members, meta_type_id=meta_type_id, properties=properties, provider_display_name=provider_display_name, resource_version=resource_version, subject_descriptor=subject_descriptor, unique_user_id=unique_user_id)


__all__ = [
    'AccountEntitlement',
    'AccountEntitlementUpdateModel',
    'AccountLicenseExtensionUsage',
    'AccountLicenseUsage',
    'AccountRights',
    'AccountUserLicense',
    'ClientRightsContainer',
    'ExtensionAssignment',
    'ExtensionAssignmentDetails',
    'ExtensionLicenseData',
    'ExtensionOperationResult',
    'ExtensionRightsResult',
    'ExtensionSource',
    'GraphSubjectBase',
    'IdentityBase',
    'IdentityMapping',
    'IdentityRef',
    'License',
    'MsdnEntitlement',
    'ReferenceLinks',
    'Identity',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .graph_client import GraphClient

__all__ = [
    'Avatar',
    'GraphCachePolicies',
    'GraphDescriptorResult',
    'GraphFederatedProviderData',
    'GraphGroup',
    'GraphGroupCreationContext',
    'GraphMember',
    'GraphMembership',
    'GraphMembershipState',
    'GraphMembershipTraversal',
    'GraphProviderInfo',
    'GraphScope',
    'GraphScopeCreationContext',
    'GraphStorageKeyResult',
    'GraphSubject',
    'GraphSubjectBase',
    'GraphSubjectLookup',
    'GraphSubjectLookupKey',
    'GraphSubjectQuery',
    'GraphUser',
    'GraphUserCreationContext',
    'GraphUserUpdateContext',
    'JsonPatchOperation',
    'PagedGraphGroups',
    'PagedGraphUsers',
    'ReferenceLinks',
    'GraphClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class AccountEntitlement(Model):
    """AccountEntitlement.

    :param account_id: Gets or sets the id of the account to which the license belongs
    :type account_id: str
    :param assignment_date: Gets or sets the date the license was assigned
    :type assignment_date: datetime
    :param assignment_source: Assignment Source
    :type assignment_source: object
    :param date_created: Gets or sets the creation date of the user in this account
    :type date_created: datetime
    :param last_accessed_date: Gets or sets the date of the user last sign-in to this account
    :type last_accessed_date: datetime
    :param license:
    :type license: :class:`License <azure.devops.v5_0.licensing.models.License>`
    :param origin: Licensing origin
    :type origin: object
    :param rights: The computed rights of this user in the account.
    :type rights: :class:`AccountRights <azure.devops.v5_0.licensing.models.AccountRights>`
    :param status: The status of the user in the account
    :type status: object
    :param user: Identity information of the user to which the license belongs
    :type user: :class:`IdentityRef <azure.devops.v5_0.licensing.models.IdentityRef>`
    :param user_id: Gets the id of the user to which the license belongs
    :type user_id: str
    """

    _attribute_map = {
        'account_id': {'key': 'accountId', 'type': 'str'},
        'assignment_date': {'key': 'assignmentDate', 'type': 'iso-8601'},
        'assignment_source': {'key': 'assignmentSource', 'type': 'object'},
        'date_created': {'key': 'dateCreated', 'type': 'iso-8601'},
        'last_accessed_date': {'key': 'lastAccessedDate', 'type': 'iso-8601'},
        'license': {'key': 'license', 'type': 'License'},
        'origin': {'key': 'origin', 'type': 'object'},
        'rights': {'key': 'rights', 'type': 'AccountRights'},
        'status': {'key': 'status', 'type': 'object'},
        'user': {'key': 'user', 'type': 'IdentityRef'},
        'user_id': {'key': 'userId', 'type': 'str'}
    }

    def __init__(self, account_id=None, assignment_date=None, assignment_source=None, date_created=None, last_accessed_date=None, license=None, origin=None, rights=None, status=None, user=None, user_id=None):
        super(AccountEntitlement, self).__init__()
        self.account_id = account_id
        self.assignment_date = assignment_date
        self.assignment_source = assignment_source
        self.date_created = date_created
        self.last_accessed_date = last_accessed_date
        self.license = license
        self.origin = origin
        self.rights = rights
        self.status = status
        self.user = user
        self.user_id = user_id


class AccountEntitlementUpdateModel(Model):
    """AccountEntitlementUpdateModel.

    :param license: Gets or sets the license for the entitlement
    :type license: :class:`License <azure.devops.v5_0.licensing.models.License>`
    """

    _attribute_map = {
        'license': {'key': 'license', 'type': 'License'}
    }

    def __init__(self, license=None):
        super(AccountEntitlementUpdateModel, self).__init__()
        self.license = license


class AccountLicenseExtensionUsage(Model):
    """AccountLicenseExtensionUsage.

    :param extension_id:
    :type extension_id: str
    :param extension_name:
    :type extension_name: str
    :param included_quantity:
    :type included_quantity: int
    :param is_trial:
    :type is_trial: bool
    :param minimum_license_required:
    :type minimum_license_required: object
    :param msdn_used_count:
    :type msdn_used_count: int
    :param provisioned_count:
    :type provisioned_count: int
    :param remaining_trial_days:
    :type remaining_trial_days: int
    :param trial_expiry_date:
    :type trial_expiry_date: datetime
    :param used_count:
    :type used_count: int
    """

    _attribute_map = {
        'extension_id': {'key': 'extensionId', 'type': 'str'},
        'extension_name': {'key': 'extensionName', 'type': 'str'},
        'included_quantity': {'key': 'includedQuantity', 'type': 'int'},
        'is_trial': {'key': 'isTrial', 'type': 'bool'},
        'minimum_license_required': {'key': 'minimumLicenseRequired', 'type': 'object'},
        'msdn_used_count': {'key': 'msdnUsedCount', 'type': 'int'},
        'provisioned_count': {'key': 'provisionedCount', 'type': 'int'},
        'remaining_trial_days': {'key': 'remainingTrialDays', 'type': 'int'},
        'trial_expiry_date': {'key': 'trialExpiryDate', 'type': 'iso-8601'},
        'used_count': {'key': 'usedCount', 'type': 'int'}
    }

    def __init__(self, extension_id=None, extension_name=None, included_quantity=None, is_trial=None, minimum_license_required=None, msdn_used_count=None, provisioned_count=None, remaining_trial_days=None, trial_expiry_date=None, used_count=None):
        super(AccountLicenseExtensionUsage, self).__init__()
        self.extension_id = extension_id
        self.extension_name = extension_name
        self.included_quantity = included_quantity
        self.is_trial = is_trial
        self.minimum_license_required = minimum_license_required
        self.msdn_used_count = msdn_used_count
        self.provisioned_count = provisioned_count
        self.remaining_trial_days = remaining_trial_days
        self.trial_expiry_date = trial_expiry_date
        self.used_count = used_count


class AccountLicenseUsage(Model):
    """AccountLicenseUsage.

    :param disabled_count: Amount that is disabled (Usually from licenses that were provisioned, but became invalid due to loss of subscription in a new billing cycle)
    :type disabled_count: int
    :param license:
    :type license: :class:`AccountUserLicense <azure.devops.v5_0.licensing.models.AccountUserLicense>`
    :param pending_provisioned_count: Amount that will be purchased in the next billing cycle
    :type pending_provisioned_count: int
    :param provisioned_count: Amount that has been purchased
    :type provisioned_count: int
    :param used_count: Amount currently being used.
    :type used_count: int
    """

    _attribute_map = {
        'disabled_count': {'key': 'disabledCount', 'type': 'int'},
        'license': {'key': 'license', 'type': 'AccountUserLicense'},
        'pending_provisioned_count': {'key': 'pendingProvisionedCount', 'type': 'int'},
        'provisioned_count': {'key': 'provisionedCount', 'type': 'int'},
        'used_count': {'key': 'usedCount', 'type': 'int'}
    }

    def __init__(self, disabled_count=None, license=None, pending_provisioned_count=None, provisioned_count=None, used_count=None):
        super(AccountLicenseUsage, self).__init__()
        self.disabled_count = disabled_count
        self.license = license
        self.pending_provisioned_count = pending_provisioned_count
        self.provisioned_count = provisioned_count
        self.used_count = used_count


class AccountRights(Model):
    """AccountRights.

    :param level:
    :type level: object
    :param reason:
    :type reason: str
    """

    _attribute_map = {
        'level': {'key': 'level', 'type': 'object'},
        'reason': {'key': 'reason', 'type': 'str'}
    }

    def __init__(self, level=None, reason=None):
        super(AccountRights, self).__init__()
        self.level = level
        self.reason = reason


class AccountUserLicense(Model):
    """AccountUserLicense.

    :param license:
    :type license: int
    :param source:
    :type source: object
    """

    _attribute_map = {
        'license': {'key': 'license', 'type': 'int'},
        'source': {'key': 'source', 'type': 'object'}
    }

    def __init__(self, license=None, source=None):
        super(AccountUserLicense, self).__init__()
        self.license = license
        self.source = source


class ClientRightsContainer(Model):
    """ClientRightsContainer.

    :param certificate_bytes:
    :type certificate_bytes: str
    :param token:
    :type token: str
    """

    _attribute_map = {
        'certificate_bytes': {'key': 'certificateBytes', 'type': 'str'},
        'token': {'key': 'token', 'type': 'str'}
    }

    def __init__(self, certificate_bytes=None, token=None):
        super(ClientRightsContainer, self).__init__()
        self.certificate_bytes = certificate_bytes
        self.token = token


class ExtensionAssignment(Model):
    """ExtensionAssignment.

    :param extension_gallery_id: Gets or sets the extension ID to assign.
    :type extension_gallery_id: str
    :param is_auto_assignment: Set to true if this a auto assignment scenario.
    :type is_auto_assignment: bool
    :param licensing_source: Gets or sets the licensing source.
    :type licensing_source: object
    :param user_ids: Gets or sets the user IDs to assign the extension to.
    :type user_ids: list of str
    """

    _attribute_map = {
        'extension_gallery_id': {'key': 'extensionGalleryId', 'type': 'str'},
        'is_auto_assignment': {'key': 'isAutoAssignment', 'type': 'bool'},
        'licensing_source': {'key': 'licensingSource', 'type': 'object'},
        'user_ids': {'key': 'userIds', 'type': '[str]'}
    }

    def __init__(self, extension_gallery_id=None, is_auto_assignment=None, licensing_source=None, user_ids=None):
        super(ExtensionAssignment, self).__init__()
        self.extension_gallery_id = extension_gallery_id
        self.is_auto_assignment = is_auto_assignment
        self.licensing_source = licensing_source
        self.user_ids = user_ids


class ExtensionAssignmentDetails(Model):
    """ExtensionAssignmentDetails.

    :param assignment_status:
    :type assignment_status: object
    :param source_collection_name:
    :type source_collection_name: str
    """

    _attribute_map = {
        'assignment_status': {'key': 'assignmentStatus', 'type': 'object'},
        'source_collection_name': {'key': 'sourceCollectionName', 'type': 'str'}
    }

    def __init__(self, assignment_status=None, source_collection_name=None):
        super(ExtensionAssignmentDetails, self).__init__()
        self.assignment_status = assignment_status
        self.source_collection_name = source_collection_name


class ExtensionLicenseData(Model):
    """ExtensionLicenseData.

    :param created_date:
    :type created_date: datetime
    :param extension_id:
    :type extension_id: str
    :param is_free:
    :type is_free: bool
    :param minimum_required_access_level:
    :type minimum_required_access_level: object
    :param updated_date:
    :type updated_date: datetime
    """

    _attribute_map = {
        'created_date': {'key': 'createdDate', 'type': 'iso-8601'},
        'extension_id': {'key': 'extensionId', 'type': 'str'},
        'is_free': {'key': 'isFree', 'type': 'bool'},
        'minimum_required_access_level': {'key': 'minimumRequiredAccessLevel', 'type': 'object'},
        'updated_date': {'key': 'updatedDate', 'type': 'iso-8601'}
    }

    def __init__(self, created_date=None, extension_id=None, is_free=None, minimum_required_access_level=None, updated_date=None):
        super(ExtensionLicenseData, self).__init__()
        self.created_date = created_date
        self.extension_id = extension_id
        self.is_free = is_free
        self.minimum_required_access_level = minimum_required_access_level
        self.updated_date = updated_date


class ExtensionOperationResult(Model):
    """ExtensionOperationResult.

    :param account_id:
    :type account_id: str
    :param extension_id:
    :type extension_id: str
    :param message:
    :type message: str
    :param operation:
    :type operation: object
    :param result:
    :type result: object
    :param user_id:
    :type user_id: str
    """

    _attribute_map = {
        'account_id': {'key': 'accountId', 'type': 'str'},
        'extension_id': {'key': 'extensionId', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'operation': {'key': 'operation', 'type': 'object'},
        'result': {'key': 'result', 'type': 'object'},
        'user_id': {'key': 'userId', 'type': 'str'}
    }

    def __init__(self, account_id=None, extension_id=None, message=None, operation=None, result=None, user_id=None):
        super(ExtensionOperationResult, self).__init__()
        self.account_id = account_id
        self.extension_id = extension_id
        self.message = message
        self.operation = operation
        self.result = result
        self.user_id = user_id


class ExtensionRightsResult(Model):
    """ExtensionRightsResult.

    :param entitled_extensions:
    :type entitled_extensions: list of str
    :param host_id:
    :type host_id: str
    :param reason:
    :type reason: str
    :param reason_code:
    :type reason_code: object
    :param result_code:
    :type result_code: object
    """

    _attribute_map = {
        'entitled_extensions': {'key': 'entitledExtensions', 'type': '[str]'},
        'host_id': {'key': 'hostId', 'type': 'str'},
        'reason': {'key': 'reason', 'type': 'str'},
        'reason_code': {'key': 'reasonCode', 'type': 'object'},
        'result_code': {'key': 'resultCode', 'type': 'object'}
    }

    def __init__(self, entitled_extensions=None, host_id=None, reason=None, reason_code=None, result_code=None):
        super(ExtensionRightsResult, self).__init__()
        self.entitled_extensions = entitled_extensions
        self.host_id = host_id
        self.reason = reason
        self.reason_code = reason_code
        self.result_code = result_code


class ExtensionSource(Model):
    """ExtensionSource.

    :param assignment_source: Assignment Source
    :type assignment_source: object
    :param extension_gallery_id: extension Identifier
    :type extension_gallery_id: str
    :param licensing_source: The licensing source of the extension. Account, Msdn, ect.
    :type licensing_source: object
    """

    _attribute_map = {
        'assignment_source': {'key': 'assignmentSource', 'type': 'object'},
        'extension_gallery_id': {'key': 'extensionGalleryId', 'type': 'str'},
        'licensing_source': {'key': 'licensingSource', 'type': 'object'}
    }

    def __init__(self, assignment_source=None, extension_gallery_id=None, licensing_source=None):
        super(ExtensionSource, self).__init__()
        self.assignment_source = assignment_source
        self.extension_gallery_id = extension_gallery_id
        self.licensing_source = licensing_source


class GraphSubjectBase(Model):
    """GraphSubjectBase.

    :param _links: This field contains zero or more interesting links about the graph subject. These links may be invoked to obtain additional relationships or more detailed information about this graph subject.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.microsoft._visual_studio._services._web_api.models.ReferenceLinks>`
    :param descriptor: The descriptor is the primary way to reference the graph subject while the system is running. This field will uniquely identify the same graph subject across both Accounts and Organizations.
    :type descriptor: str
    :param display_name: This is the non-unique display name of the graph subject. To change this field, you must alter its value in the source provider.
    :type display_name: str
    :param url: This url is the full route to the source resource of this graph subject.
    :type url: str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, _links=None, descriptor=None, display_name=None, url=None):
        super(GraphSubjectBase, self).__init__()
        self._links = _links
        self.descriptor = descriptor
        self.display_name = display_name
        self.url = url


class IdentityRef(GraphSubjectBase):
    """IdentityRef.

    :param _links: This field contains zero or more interesting links about the graph subject. These links may be invoked to obtain additional relationships or more detailed information about this graph subject.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.microsoft._visual_studio._services._web_api.models.ReferenceLinks>`
    :param descriptor: The descriptor is the primary way to reference the graph subject while the system is running. This field will uniquely identify the same graph subject across both Accounts and Organizations.
    :type descriptor: str
    :param display_name: This is the non-unique display name of the graph subject. To change this field, you must alter its value in the source provider.
    :type display_name: str
    :param url: This url is the full route to the source resource of this graph subject.
    :type url: str
    :param directory_alias:
    :type directory_alias: str
    :param id:
    :type id: str
    :param image_url:
    :type image_url: str
    :param inactive:
    :type inactive: bool
    :param is_aad_identity:
    :type is_aad_identity: bool
    :param is_container:
    :type is_container: bool
    :param is_deleted_in_origin:
    :type is_deleted_in_origin: bool
    :param profile_url:
    :type profile_url: str
    :param unique_name:
    :type unique_name: str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'directory_alias': {'key': 'directoryAlias', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'image_url': {'key': 'imageUrl', 'type': 'str'},
        'inactive': {'key': 'inactive', 'type': 'bool'},
        'is_aad_identity': {'key': 'isAadIdentity', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'is_deleted_in_origin': {'key': 'isDeletedInOrigin', 'type': 'bool'},
        'profile_url': {'key': 'profileUrl', 'type': 'str'},
        'unique_name': {'key': 'uniqueName', 'type': 'str'}
    }

    def __init__(self, _links=None, descriptor=None, display_name=None, url=None, directory_alias=None, id=None, image_url=None, inactive=None, is_aad_identity=None, is_container=None, is_deleted_in_origin=None, profile_url=None, unique_name=None):
        super(IdentityRef, self).__init__(_links=_links, descriptor=descriptor, display_name=display_name, url=url)
        self.directory_alias = directory_alias
        self.id = id
        self.image_url = image_url
        self.inactive = inactive
        self.is_aad_identity = is_aad_identity
        self.is_container = is_container
        self.is_deleted_in_origin = is_deleted_in_origin
        self.profile_url = profile_url
        self.unique_name = unique_name


class License(Model):
    """License.

    :param source: Gets the source of the license
    :type source: object
    """

    _attribute_map = {
        'source': {'key': 'source', 'type': 'object'}
    }

    def __init__(self, source=None):
        super(License, self).__init__()
        self.source = source


class MsdnEntitlement(Model):
    """MsdnEntitlement.

    :param entitlement_code: Entilement id assigned to Entitlement in Benefits Database.
    :type entitlement_code: str
    :param entitlement_name: Entitlement Name e.g. Downloads, Chat.
    :type entitlement_name: str
    :param entitlement_type: Type of Entitlement e.g. Downloads, Chat.
    :type entitlement_type: str
    :param is_activated: Entitlement activation status
    :type is_activated: bool
    :param is_entitlement_available: Entitlement availability
    :type is_entitlement_available: bool
    :param subscription_channel: Write MSDN Channel into CRCT (Retail,MPN,VL,BizSpark,DreamSpark,MCT,FTE,Technet,WebsiteSpark,Other)
    :type subscription_channel: str
    :param subscription_expiration_date: Subscription Expiration Date.
    :type subscription_expiration_date: datetime
    :param subscription_id: Subscription id which identifies the subscription itself. This is the Benefit Detail Guid from BMS.
    :type subscription_id: str
    :param subscription_level_code: Identifier of the subscription or benefit level.
    :type subscription_level_code: str
    :param subscription_level_name: Name of subscription level.
    :type subscription_level_name: str
    :param subscription_status: Subscription Status Code (ACT, PND, INA ...).
    :type subscription_status: str
    """

    _attribute_map = {
        'entitlement_code': {'key': 'entitlementCode', 'type': 'str'},
        'entitlement_name': {'key': 'entitlementName', 'type': 'str'},
        'entitlement_type': {'key': 'entitlementType', 'type': 'str'},
        'is_activated': {'key': 'isActivated', 'type': 'bool'},
        'is_entitlement_available': {'key': 'isEntitlementAvailable', 'type': 'bool'},
        'subscription_channel': {'key': 'subscriptionChannel', 'type': 'str'},
        'subscription_expiration_date': {'key': 'subscriptionExpirationDate', 'type': 'iso-8601'},
        'subscription_id': {'key': 'subscriptionId', 'type': 'str'},
        'subscription_level_code': {'key': 'subscriptionLevelCode', 'type': 'str'},
        'subscription_level_name': {'key': 'subscriptionLevelName', 'type': 'str'},
        'subscription_status': {'key': 'subscriptionStatus', 'type': 'str'}
    }

    def __init__(self, entitlement_code=None, entitlement_name=None, entitlement_type=None, is_activated=None, is_entitlement_available=None, subscription_channel=None, subscription_expiration_date=None, subscription_id=None, subscription_level_code=None, subscription_level_name=None, subscription_status=None):
        super(MsdnEntitlement, self).__init__()
        self.entitlement_code = entitlement_code
        self.entitlement_name = entitlement_name
        self.entitlement_type = entitlement_type
        self.is_activated = is_activated
        self.is_entitlement_available = is_entitlement_available
        self.subscription_channel = subscription_channel
        self.subscription_expiration_date = subscription_expiration_date
        self.subscription_id = subscription_id
        self.subscription_level_code = subscription_level_code
        self.subscription_level_name = subscription_level_name
        self.subscription_status = subscription_status


class ReferenceLinks(Model):
    """ReferenceLinks.

    :param links: The readonly view of the links.  Because Reference links are readonly, we only want to expose them as read only.
    :type links: dict
    """

    _attribute_map = {
        'links': {'key': 'links', 'type': '{object}'}
    }

    def __init__(self, links=None):
        super(ReferenceLinks, self).__init__()
        self.links = links


__all__ = [
    'AccountEntitlement',
    'AccountEntitlementUpdateModel',
    'AccountLicenseExtensionUsage',
    'AccountLicenseUsage',
    'AccountRights',
    'AccountUserLicense',
    'ClientRightsContainer',
    'ExtensionAssignment',
    'ExtensionAssignmentDetails',
    'ExtensionLicenseData',
    'ExtensionOperationResult',
    'ExtensionRightsResult',
    'ExtensionSource',
    'GraphSubjectBase',
    'IdentityRef',
    'License',
    'MsdnEntitlement',
    'ReferenceLinks',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .notification_client import NotificationClient

__all__ = [
    'ArtifactFilter',
    'BaseSubscriptionFilter',
    'BatchNotificationOperation',
    'EventActor',
    'EventScope',
    'EventsEvaluationResult',
    'EventTransformRequest',
    'EventTransformResult',
    'ExpressionFilterClause',
    'ExpressionFilterGroup',
    'ExpressionFilterModel',
    'FieldInputValues',
    'FieldValuesQuery',
    'GraphSubjectBase',
    'IdentityRef',
    'INotificationDiagnosticLog',
    'InputValue',
    'InputValues',
    'InputValuesError',
    'InputValuesQuery',
    'ISubscriptionChannel',
    'ISubscriptionFilter',
    'NotificationAdminSettings',
    'NotificationAdminSettingsUpdateParameters',
    'NotificationDiagnosticLogMessage',
    'NotificationEventField',
    'NotificationEventFieldOperator',
    'NotificationEventFieldType',
    'NotificationEventPublisher',
    'NotificationEventRole',
    'NotificationEventType',
    'NotificationEventTypeCategory',
    'NotificationQueryCondition',
    'NotificationReason',
    'NotificationsEvaluationResult',
    'NotificationStatistic',
    'NotificationStatisticsQuery',
    'NotificationStatisticsQueryConditions',
    'NotificationSubscriber',
    'NotificationSubscriberUpdateParameters',
    'NotificationSubscription',
    'NotificationSubscriptionCreateParameters',
    'NotificationSubscriptionTemplate',
    'NotificationSubscriptionUpdateParameters',
    'OperatorConstraint',
    'ReferenceLinks',
    'SubscriptionAdminSettings',
    'SubscriptionChannelWithAddress',
    'SubscriptionDiagnostics',
    'SubscriptionEvaluationRequest',
    'SubscriptionEvaluationResult',
    'SubscriptionEvaluationSettings',
    'SubscriptionManagement',
    'SubscriptionQuery',
    'SubscriptionQueryCondition',
    'SubscriptionScope',
    'SubscriptionTracing',
    'SubscriptionUserSettings',
    'UpdateSubscripitonDiagnosticsParameters',
    'UpdateSubscripitonTracingParameters',
    'ValueDefinition',
    'VssNotificationEvent',
    'NotificationClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .pipeline_permissions_client import PipelinePermissionsClient

__all__ = [
    'GraphSubjectBase',
    'IdentityRef',
    'Permission',
    'PipelinePermission',
    'ReferenceLinks',
    'Resource',
    'ResourcePipelinePermissions',
    'PipelinePermissionsClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .audit_client import AuditClient

__all__ = [
    'AuditActionInfo',
    'AuditLogEntry',
    'AuditLogQueryResult',
    'AuditStream',
    'DecoratedAuditLogEntry',
    'AuditClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest

from azure_devtools.scenario_tests import AllowLargeResponse
from .utilities.helper import (
    DevopsScenarioTest, get_random_name, disable_telemetry, set_authentication, get_test_org_from_env_variable)

DEVOPS_CLI_TEST_ORGANIZATION = get_test_org_from_env_variable() or 'Https://dev.azure.com/devops-cli-test-org'

class ReposRepoTests(DevopsScenarioTest):
    @AllowLargeResponse(size_kb=3072)
    @disable_telemetry
    @set_authentication
    def test_repos_createListShowDelete(self):
        random_name = get_random_name(8)
        self.cmd('az devops configure --defaults organization=' + DEVOPS_CLI_TEST_ORGANIZATION)
        
        created_repo_id = None
        try:
            create_repo_command = 'az repos create --name ' + random_name +' --project RepoCreateListShowDeleteTests --output json --detect false'
            repo_create_output = self.cmd(create_repo_command).get_output_in_json()
            created_repo_id = repo_create_output["id"]
            created_repo_name = repo_create_output["name"]
            assert len(created_repo_id) > 0

            list_repo_command = 'az repos list --project RepoCreateListShowDeleteTests --output json  --detect false'
            verified_repo_list = False
            list_repo_output_before_delete = self.cmd(list_repo_command).get_output_in_json()
            for repos in list_repo_output_before_delete:
                if(repos["id"] == created_repo_id):
                    verified_repo_list = True
            assert verified_repo_list == True

            show_repo_command = 'az repos show -r ' + created_repo_id + ' --project RepoCreateListShowDeleteTests --output json --detect false'
            show_repo_output = self.cmd(show_repo_command).get_output_in_json()
            assert show_repo_output["id"] == created_repo_id
            
            updated_repo_name = created_repo_name + 'Updated'
            update_repo_command = 'az repos update -r ' + created_repo_id + ' -p RepoCreateListShowDeleteTests --name ' + updated_repo_name  + ' -o json --detect false'
            update_repo_output = self.cmd(update_repo_command).get_output_in_json()
            assert update_repo_output["id"] == created_repo_id
            assert update_repo_output["name"] == updated_repo_name

        finally:
            delete_repo_command = 'az repos delete --id ' + created_repo_id + ' --project RepoCreateListShowDeleteTests -y --output json --detect false'
            self.cmd(delete_repo_command)
            
            #Verify Deletion
            list_repo_output_after_delete = self.cmd(list_repo_command).get_output_in_json()
            for repos in list_repo_output_after_delete:
                if(repos["id"] == created_repo_id):
                    assert 0
                    


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------


class ClientFactoryV5_0(object):
    """ClientFactoryV5_0.
    A factory class to get the 5.0 clients.
    """

    def __init__(self, connection):
        self._connection = connection

    def get_accounts_client(self):
        """get_accounts_client.
        Gets the 5.0 version of the AccountsClient
        :rtype: :class:`<AccountsClient> <azure.devops.v5_0.accounts.accounts_client.AccountsClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.accounts.accounts_client.AccountsClient')

    def get_boards_client(self):
        """get_boards_client.
        Gets the 5.0 version of the BoardsClient
        :rtype: :class:`<BoardsClient> <azure.devops.v5_0.boards.boards_client.BoardsClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.boards.boards_client.BoardsClient')

    def get_build_client(self):
        """get_build_client.
        Gets the 5.0 version of the BuildClient
        :rtype: :class:`<BuildClient> <azure.devops.v5_0.build.build_client.BuildClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.build.build_client.BuildClient')

    def get_client_trace_client(self):
        """get_client_trace_client.
        Gets the 5.0 version of the ClientTraceClient
        :rtype: :class:`<ClientTraceClient> <azure.devops.v5_0.client_trace.client_trace_client.ClientTraceClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.client_trace.client_trace_client.ClientTraceClient')

    def get_cloud_load_test_client(self):
        """get_cloud_load_test_client.
        Gets the 5.0 version of the CloudLoadTestClient
        :rtype: :class:`<CloudLoadTestClient> <azure.devops.v5_0.cloud_load_test.cloud_load_test_client.CloudLoadTestClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.cloud_load_test.cloud_load_test_client.CloudLoadTestClient')

    def get_contributions_client(self):
        """get_contributions_client.
        Gets the 5.0 version of the ContributionsClient
        :rtype: :class:`<ContributionsClient> <azure.devops.v5_0.contributions.contributions_client.ContributionsClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.contributions.contributions_client.ContributionsClient')

    def get_core_client(self):
        """get_core_client.
        Gets the 5.0 version of the CoreClient
        :rtype: :class:`<CoreClient> <azure.devops.v5_0.core.core_client.CoreClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.core.core_client.CoreClient')

    def get_customer_intelligence_client(self):
        """get_customer_intelligence_client.
        Gets the 5.0 version of the CustomerIntelligenceClient
        :rtype: :class:`<CustomerIntelligenceClient> <azure.devops.v5_0.customer_intelligence.customer_intelligence_client.CustomerIntelligenceClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.customer_intelligence.customer_intelligence_client.CustomerIntelligenceClient')

    def get_dashboard_client(self):
        """get_dashboard_client.
        Gets the 5.0 version of the DashboardClient
        :rtype: :class:`<DashboardClient> <azure.devops.v5_0.dashboard.dashboard_client.DashboardClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.dashboard.dashboard_client.DashboardClient')

    def get_extension_management_client(self):
        """get_extension_management_client.
        Gets the 5.0 version of the ExtensionManagementClient
        :rtype: :class:`<ExtensionManagementClient> <azure.devops.v5_0.extension_management.extension_management_client.ExtensionManagementClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.extension_management.extension_management_client.ExtensionManagementClient')

    def get_feature_availability_client(self):
        """get_feature_availability_client.
        Gets the 5.0 version of the FeatureAvailabilityClient
        :rtype: :class:`<FeatureAvailabilityClient> <azure.devops.v5_0.feature_availability.feature_availability_client.FeatureAvailabilityClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.feature_availability.feature_availability_client.FeatureAvailabilityClient')

    def get_feature_management_client(self):
        """get_feature_management_client.
        Gets the 5.0 version of the FeatureManagementClient
        :rtype: :class:`<FeatureManagementClient> <azure.devops.v5_0.feature_management.feature_management_client.FeatureManagementClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.feature_management.feature_management_client.FeatureManagementClient')

    def get_feed_client(self):
        """get_feed_client.
        Gets the 5.0 version of the FeedClient
        :rtype: :class:`<FeedClient> <azure.devops.v5_0.feed.feed_client.FeedClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.feed.feed_client.FeedClient')

    def get_feed_token_client(self):
        """get_feed_token_client.
        Gets the 5.0 version of the FeedTokenClient
        :rtype: :class:`<FeedTokenClient> <azure.devops.v5_0.feed_token.feed_token_client.FeedTokenClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.feed_token.feed_token_client.FeedTokenClient')

    def get_file_container_client(self):
        """get_file_container_client.
        Gets the 5.0 version of the FileContainerClient
        :rtype: :class:`<FileContainerClient> <azure.devops.v5_0.file_container.file_container_client.FileContainerClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.file_container.file_container_client.FileContainerClient')

    def get_gallery_client(self):
        """get_gallery_client.
        Gets the 5.0 version of the GalleryClient
        :rtype: :class:`<GalleryClient> <azure.devops.v5_0.gallery.gallery_client.GalleryClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.gallery.gallery_client.GalleryClient')

    def get_git_client(self):
        """get_git_client.
        Gets the 5.0 version of the GitClient
        :rtype: :class:`<GitClient> <azure.devops.v5_0.git.git_client.GitClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.git.git_client.GitClient')

    def get_graph_client(self):
        """get_graph_client.
        Gets the 5.0 version of the GraphClient
        :rtype: :class:`<GraphClient> <azure.devops.v5_0.graph.graph_client.GraphClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.graph.graph_client.GraphClient')

    def get_identity_client(self):
        """get_identity_client.
        Gets the 5.0 version of the IdentityClient
        :rtype: :class:`<IdentityClient> <azure.devops.v5_0.identity.identity_client.IdentityClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.identity.identity_client.IdentityClient')

    def get_licensing_client(self):
        """get_licensing_client.
        Gets the 5.0 version of the LicensingClient
        :rtype: :class:`<LicensingClient> <azure.devops.v5_0.licensing.licensing_client.LicensingClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.licensing.licensing_client.LicensingClient')

    def get_location_client(self):
        """get_location_client.
        Gets the 5.0 version of the LocationClient
        :rtype: :class:`<LocationClient> <azure.devops.v5_0.location.location_client.LocationClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.location.location_client.LocationClient')

    def get_maven_client(self):
        """get_maven_client.
        Gets the 5.0 version of the MavenClient
        :rtype: :class:`<MavenClient> <azure.devops.v5_0.maven.maven_client.MavenClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.maven.maven_client.MavenClient')

    def get_member_entitlement_management_client(self):
        """get_member_entitlement_management_client.
        Gets the 5.0 version of the MemberEntitlementManagementClient
        :rtype: :class:`<MemberEntitlementManagementClient> <azure.devops.v5_0.member_entitlement_management.member_entitlement_management_client.MemberEntitlementManagementClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.member_entitlement_management.member_entitlement_management_client.MemberEntitlementManagementClient')

    def get_notification_client(self):
        """get_notification_client.
        Gets the 5.0 version of the NotificationClient
        :rtype: :class:`<NotificationClient> <azure.devops.v5_0.notification.notification_client.NotificationClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.notification.notification_client.NotificationClient')

    def get_npm_client(self):
        """get_npm_client.
        Gets the 5.0 version of the NpmClient
        :rtype: :class:`<NpmClient> <azure.devops.v5_0.npm.npm_client.NpmClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.npm.npm_client.NpmClient')

    def get_nuget_client(self):
        """get_nuget_client.
        Gets the 5.0 version of the NuGetClient
        :rtype: :class:`<NuGetClient> <azure.devops.v5_0.nuget.nuget_client.NuGetClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.nuget.nuget_client.NuGetClient')

    def get_operations_client(self):
        """get_operations_client.
        Gets the 5.0 version of the OperationsClient
        :rtype: :class:`<OperationsClient> <azure.devops.v5_0.operations.operations_client.OperationsClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.operations.operations_client.OperationsClient')

    def get_policy_client(self):
        """get_policy_client.
        Gets the 5.0 version of the PolicyClient
        :rtype: :class:`<PolicyClient> <azure.devops.v5_0.policy.policy_client.PolicyClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.policy.policy_client.PolicyClient')

    def get_profile_client(self):
        """get_profile_client.
        Gets the 5.0 version of the ProfileClient
        :rtype: :class:`<ProfileClient> <azure.devops.v5_0.profile.profile_client.ProfileClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.profile.profile_client.ProfileClient')

    def get_project_analysis_client(self):
        """get_project_analysis_client.
        Gets the 5.0 version of the ProjectAnalysisClient
        :rtype: :class:`<ProjectAnalysisClient> <azure.devops.v5_0.project_analysis.project_analysis_client.ProjectAnalysisClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.project_analysis.project_analysis_client.ProjectAnalysisClient')

    def get_provenance_client(self):
        """get_provenance_client.
        Gets the 5.0 version of the ProvenanceClient
        :rtype: :class:`<ProvenanceClient> <azure.devops.v5_0.provenance.provenance_client.ProvenanceClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.provenance.provenance_client.ProvenanceClient')

    def get_py_pi_api_client(self):
        """get_py_pi_api_client.
        Gets the 5.0 version of the PyPiApiClient
        :rtype: :class:`<PyPiApiClient> <azure.devops.v5_0.py_pi_api.py_pi_api_client.PyPiApiClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.py_pi_api.py_pi_api_client.PyPiApiClient')

    def get_release_client(self):
        """get_release_client.
        Gets the 5.0 version of the ReleaseClient
        :rtype: :class:`<ReleaseClient> <azure.devops.v5_0.release.release_client.ReleaseClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.release.release_client.ReleaseClient')

    def get_search_client(self):
        """get_search_client.
        Gets the 5.0 version of the SearchClient
        :rtype: :class:`<SearchClient> <azure.devops.v5_0.search.search_client.SearchClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.search.search_client.SearchClient')

    def get_security_client(self):
        """get_security_client.
        Gets the 5.0 version of the SecurityClient
        :rtype: :class:`<SecurityClient> <azure.devops.v5_0.security.security_client.SecurityClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.security.security_client.SecurityClient')

    def get_service_endpoint_client(self):
        """get_service_endpoint_client.
        Gets the 5.0 version of the ServiceEndpointClient
        :rtype: :class:`<ServiceEndpointClient> <azure.devops.v5_0.service_endpoint.service_endpoint_client.ServiceEndpointClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.service_endpoint.service_endpoint_client.ServiceEndpointClient')

    def get_service_hooks_client(self):
        """get_service_hooks_client.
        Gets the 5.0 version of the ServiceHooksClient
        :rtype: :class:`<ServiceHooksClient> <azure.devops.v5_0.service_hooks.service_hooks_client.ServiceHooksClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.service_hooks.service_hooks_client.ServiceHooksClient')

    def get_settings_client(self):
        """get_settings_client.
        Gets the 5.0 version of the SettingsClient
        :rtype: :class:`<SettingsClient> <azure.devops.v5_0.settings.settings_client.SettingsClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.settings.settings_client.SettingsClient')

    def get_symbol_client(self):
        """get_symbol_client.
        Gets the 5.0 version of the SymbolClient
        :rtype: :class:`<SymbolClient> <azure.devops.v5_0.symbol.symbol_client.SymbolClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.symbol.symbol_client.SymbolClient')

    def get_task_client(self):
        """get_task_client.
        Gets the 5.0 version of the TaskClient
        :rtype: :class:`<TaskClient> <azure.devops.v5_0.task.task_client.TaskClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.task.task_client.TaskClient')

    def get_task_agent_client(self):
        """get_task_agent_client.
        Gets the 5.0 version of the TaskAgentClient
        :rtype: :class:`<TaskAgentClient> <azure.devops.v5_0.task_agent.task_agent_client.TaskAgentClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.task_agent.task_agent_client.TaskAgentClient')

    def get_test_client(self):
        """get_test_client.
        Gets the 5.0 version of the TestClient
        :rtype: :class:`<TestClient> <azure.devops.v5_0.test.test_client.TestClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.test.test_client.TestClient')

    def get_tfvc_client(self):
        """get_tfvc_client.
        Gets the 5.0 version of the TfvcClient
        :rtype: :class:`<TfvcClient> <azure.devops.v5_0.tfvc.tfvc_client.TfvcClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.tfvc.tfvc_client.TfvcClient')

    def get_token_admin_client(self):
        """get_token_admin_client.
        Gets the 5.0 version of the TokenAdminClient
        :rtype: :class:`<TokenAdminClient> <azure.devops.v5_0.token_admin.token_admin_client.TokenAdminClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.token_admin.token_admin_client.TokenAdminClient')

    def get_token_administration_client(self):
        """get_token_administration_client.
        Gets the 5.0 version of the TokenAdministrationClient
        :rtype: :class:`<TokenAdministrationClient> <azure.devops.v5_0.token_administration.token_administration_client.TokenAdministrationClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.token_administration.token_administration_client.TokenAdministrationClient')

    def get_upack_api_client(self):
        """get_upack_api_client.
        Gets the 5.0 version of the UPackApiClient
        :rtype: :class:`<UPackApiClient> <azure.devops.v5_0.upack_api.upack_api_client.UPackApiClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.upack_api.upack_api_client.UPackApiClient')

    def get_upack_packaging_client(self):
        """get_upack_packaging_client.
        Gets the 5.0 version of the UPackPackagingClient
        :rtype: :class:`<UPackPackagingClient> <azure.devops.v5_0.upack_packaging.upack_packaging_client.UPackPackagingClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.upack_packaging.upack_packaging_client.UPackPackagingClient')

    def get_wiki_client(self):
        """get_wiki_client.
        Gets the 5.0 version of the WikiClient
        :rtype: :class:`<WikiClient> <azure.devops.v5_0.wiki.wiki_client.WikiClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.wiki.wiki_client.WikiClient')

    def get_work_client(self):
        """get_work_client.
        Gets the 5.0 version of the WorkClient
        :rtype: :class:`<WorkClient> <azure.devops.v5_0.work.work_client.WorkClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.work.work_client.WorkClient')

    def get_work_item_tracking_client(self):
        """get_work_item_tracking_client.
        Gets the 5.0 version of the WorkItemTrackingClient
        :rtype: :class:`<WorkItemTrackingClient> <azure.devops.v5_0.work_item_tracking.work_item_tracking_client.WorkItemTrackingClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.work_item_tracking.work_item_tracking_client.WorkItemTrackingClient')

    def get_work_item_tracking_process_client(self):
        """get_work_item_tracking_process_client.
        Gets the 5.0 version of the WorkItemTrackingProcessClient
        :rtype: :class:`<WorkItemTrackingProcessClient> <azure.devops.v5_0.work_item_tracking_process.work_item_tracking_process_client.WorkItemTrackingProcessClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.work_item_tracking_process.work_item_tracking_process_client.WorkItemTrackingProcessClient')

    def get_work_item_tracking_process_template_client(self):
        """get_work_item_tracking_process_template_client.
        Gets the 5.0 version of the WorkItemTrackingProcessTemplateClient
        :rtype: :class:`<WorkItemTrackingProcessTemplateClient> <azure.devops.v5_0.work_item_tracking_process_template.work_item_tracking_process_template_client.WorkItemTrackingProcessTemplateClient>`
        """
        return self._connection.get_client('azure.devops.v5_0.work_item_tracking_process_template.work_item_tracking_process_template_client.WorkItemTrackingProcessTemplateClient')




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch

from azext_devops.dev.common.telemetry import (set_tracking_data, try_send_telemetry_data)


class TestTelemetryMethods(unittest.TestCase):
    
    _TEST_DEVOPS_ORGANIZATION = 'https://dev.azure.com/AzureDevOpsCliTest'
   
    def test_send_telemetry_should_send_if_enabled(self):
        with patch('azext_devops.dev.common.telemetry._is_telemetry_enabled') as mock_telemetry_enabled:  
            mock_telemetry_enabled.return_value = True
            with patch('azext_devops.dev.common.telemetry._try_send_tracking_ci_event_async') as mock_telemetry_send:
                try_send_telemetry_data(self._TEST_DEVOPS_ORGANIZATION)
                #assert
                mock_telemetry_send.assert_called_once_with(self._TEST_DEVOPS_ORGANIZATION)


    def test_send_telemetry_should_not_send_if_disabled(self):
        with patch('azext_devops.dev.common.telemetry._is_telemetry_enabled') as mock_telemetry_enabled:  
            mock_telemetry_enabled.return_value = False
            with patch('azext_devops.dev.common.telemetry._try_send_tracking_ci_event_async') as mock_telemetry_send:
                try_send_telemetry_data(self._TEST_DEVOPS_ORGANIZATION)
                #assert
                mock_telemetry_send.assert_not_called()


    def test_set_tracking_data_should_set_data_correctly_with_command(self):
        class FakeTelemetryClass:
            def __init__(self, command):
                self.command = command

            def __iter__():
                for key, value in self.command:
                    yield(key, value)

        command_dict = FakeTelemetryClass('devops repos list')
        set_tracking_data(**{'args': command_dict})
        from azext_devops.dev.common.telemetry import vsts_tracking_data
        assert vsts_tracking_data.area == 'AzureDevopsCli'
        assert vsts_tracking_data.feature == 'devops'
        assert vsts_tracking_data.properties['Command'] == 'repos list'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azext_devops.devops_sdk.v5_0.graph.models import (GraphGroupCreationContext)


class GraphGroupVstsCreationContext(GraphGroupCreationContext):
    """GraphGroupVstsCreationContext.

    :param displayName: Name of the group.
    :type displayName: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'}
    }

    def __init__(self, display_name, description=None):
        super(GraphGroupVstsCreationContext, self).__init__()
        self.display_name = display_name
        self.description = description


class GraphGroupMailAddressCreationContext(GraphGroupCreationContext):
    """GraphGroupMailAddressCreationContext.

    :param mailAddress: This should be the mail address or the group in the source AD or AAD provider.
    :type mailAddress: str
    """

    _attribute_map = {
        'mail_address': {'key': 'mailAddress', 'type': 'str'},
    }

    def __init__(self, mail_address):
        super(GraphGroupMailAddressCreationContext, self).__init__()
        self.mail_address = mail_address


class GraphGroupOriginIdCreationContext(GraphGroupCreationContext):
    """GraphGroupOriginIdCreationContext.

    :param originId: This should be the object id or sid of the group from the source AD or AAD provider.
    :type originId: str
    """

    _attribute_map = {
        'origin_id': {'key': 'originId', 'type': 'str'},
    }

    def __init__(self, origin_id):
        super(GraphGroupOriginIdCreationContext, self).__init__()
        self.origin_id = origin_id



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from azext_devops.dev.common.arguments import should_detect


class TestArgumentsMethods(unittest.TestCase):

    def test_should_detect(self):
        # tests default behaviour for detect
        self.assertEqual(should_detect(None), True)


if __name__ == '__main__':
    unittest.main()


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import random
import string
from azure.cli.testsdk import ScenarioTest

DEVOPS_CLI_TEST_PAT_TOKEN = 'ThisShouldBeAnInvalidPatTokenWhenCheckedIn'
PAT_ENV_VARIABLE_NAME = 'AZURE_DEVOPS_EXT_PAT'
_TEST_ORG_ENV_VARIABLE_NAME = 'AZURE_DEVOPS_EXT_TEST_ORG'

class DevopsScenarioTest(ScenarioTest):
    def sleep_in_live_run(self, time_to_sleep):
        if self.in_recording or self.is_live:
            import time
            time.sleep(time_to_sleep)


def get_random_name(length):
    return ''.join(random.choice(string.ascii_letters) for m in range(length))


def disable_telemetry(test_function):
    def wrapper(*args, **kwargs):
        print("Disabling Telemetry.")
        os.environ["AZURE_CORE_COLLECT_TELEMETRY"] = "no"
        test_function(*args, **kwargs)
    return wrapper


def set_authentication(test_function):
    def wrapper(*args, **kwargs):
        print("Setting auth for test run.")
        pat_from_env = os.environ.get(PAT_ENV_VARIABLE_NAME)
        if not pat_from_env:
            print("PAT not found in environment variable. Falling back to hardcoded token.")
            os.environ[PAT_ENV_VARIABLE_NAME] = DEVOPS_CLI_TEST_PAT_TOKEN
        else:
            print("Using PAT from environment variable")
        test_function(*args, **kwargs)
    return wrapper


def get_test_org_from_env_variable():
    return os.environ.get(_TEST_ORG_ENV_VARIABLE_NAME)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azext_devops.dev.common.services import (get_settings_client,
                                              resolve_instance)


def setting_list(user_scope, key=None, organization=None, detect=None):
    organization = resolve_instance(detect=detect, organization=organization)
    client = get_settings_client(organization)
    entries = client.get_entries(user_scope=user_scope, key=key)
    return entries


def setting_add_or_update(entries, user_scope, organization=None, detect=None):
    organization = resolve_instance(detect=detect, organization=organization)
    client = get_settings_client(organization)
    client.set_entries(entries=entries, user_scope=user_scope)


def setting_remove(key, user_scope, organization=None, detect=None):
    organization = resolve_instance(detect=detect, organization=organization)
    client = get_settings_client(organization)
    client.remove_entries(key=key, user_scope=user_scope)


GLOBAL_MESSAGE_BANNERS_KEY = 'GlobalMessageBanners'
USER_SCOPE_HOST = 'host'



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function


def feedback():
    """Displays information on how to provide feedback to the Azure DevOps CLI team.
    """
    url = 'https://aka.ms/azure-devops-cli-feedback'
    print('Thank you for taking the time to share your feedback. Please submit your feedback on the following web ' +
          'page: {url}'.format(url=url))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class AccessTokenResult(Model):
    """
    :param access_token:
    :type access_token: :class:`JsonWebToken <azure.devops.v6_0.microsoft._visual_studio._services._web_api.models.JsonWebToken>`
    :param access_token_error:
    :type access_token_error: object
    :param authorization_id:
    :type authorization_id: str
    :param error_description:
    :type error_description: str
    :param has_error:
    :type has_error: bool
    :param is_first_party_client:
    :type is_first_party_client: bool
    :param refresh_token:
    :type refresh_token: :class:`RefreshTokenGrant <azure.devops.v6_0.microsoft._visual_studio._services._web_api.models.RefreshTokenGrant>`
    :param scope:
    :type scope: str
    :param token_type:
    :type token_type: str
    :param valid_to:
    :type valid_to: datetime
    """

    _attribute_map = {
        'access_token': {'key': 'accessToken', 'type': 'JsonWebToken'},
        'access_token_error': {'key': 'accessTokenError', 'type': 'object'},
        'authorization_id': {'key': 'authorizationId', 'type': 'str'},
        'error_description': {'key': 'errorDescription', 'type': 'str'},
        'has_error': {'key': 'hasError', 'type': 'bool'},
        'is_first_party_client': {'key': 'isFirstPartyClient', 'type': 'bool'},
        'refresh_token': {'key': 'refreshToken', 'type': 'RefreshTokenGrant'},
        'scope': {'key': 'scope', 'type': 'str'},
        'token_type': {'key': 'tokenType', 'type': 'str'},
        'valid_to': {'key': 'validTo', 'type': 'iso-8601'}
    }

    def __init__(self, access_token=None, access_token_error=None, authorization_id=None, error_description=None, has_error=None, is_first_party_client=None, refresh_token=None, scope=None, token_type=None, valid_to=None):
        super(AccessTokenResult, self).__init__()
        self.access_token = access_token
        self.access_token_error = access_token_error
        self.authorization_id = authorization_id
        self.error_description = error_description
        self.has_error = has_error
        self.is_first_party_client = is_first_party_client
        self.refresh_token = refresh_token
        self.scope = scope
        self.token_type = token_type
        self.valid_to = valid_to


class AuthorizationGrant(Model):
    """
    :param grant_type:
    :type grant_type: object
    """

    _attribute_map = {
        'grant_type': {'key': 'grantType', 'type': 'object'}
    }

    def __init__(self, grant_type=None):
        super(AuthorizationGrant, self).__init__()
        self.grant_type = grant_type


class ChangedIdentities(Model):
    """
    Container class for changed identities

    :param identities: Changed Identities
    :type identities: list of :class:`Identity <azure.devops.v6_0.identities.models.Identity>`
    :param more_data: More data available, set to true if pagesize is specified.
    :type more_data: bool
    :param sequence_context: Last Identity SequenceId
    :type sequence_context: :class:`ChangedIdentitiesContext <azure.devops.v6_0.identities.models.ChangedIdentitiesContext>`
    """

    _attribute_map = {
        'identities': {'key': 'identities', 'type': '[Identity]'},
        'more_data': {'key': 'moreData', 'type': 'bool'},
        'sequence_context': {'key': 'sequenceContext', 'type': 'ChangedIdentitiesContext'}
    }

    def __init__(self, identities=None, more_data=None, sequence_context=None):
        super(ChangedIdentities, self).__init__()
        self.identities = identities
        self.more_data = more_data
        self.sequence_context = sequence_context


class ChangedIdentitiesContext(Model):
    """
    Context class for changed identities

    :param group_sequence_id: Last Group SequenceId
    :type group_sequence_id: int
    :param identity_sequence_id: Last Identity SequenceId
    :type identity_sequence_id: int
    :param organization_identity_sequence_id: Last Group OrganizationIdentitySequenceId
    :type organization_identity_sequence_id: int
    :param page_size: Page size
    :type page_size: int
    """

    _attribute_map = {
        'group_sequence_id': {'key': 'groupSequenceId', 'type': 'int'},
        'identity_sequence_id': {'key': 'identitySequenceId', 'type': 'int'},
        'organization_identity_sequence_id': {'key': 'organizationIdentitySequenceId', 'type': 'int'},
        'page_size': {'key': 'pageSize', 'type': 'int'}
    }

    def __init__(self, group_sequence_id=None, identity_sequence_id=None, organization_identity_sequence_id=None, page_size=None):
        super(ChangedIdentitiesContext, self).__init__()
        self.group_sequence_id = group_sequence_id
        self.identity_sequence_id = identity_sequence_id
        self.organization_identity_sequence_id = organization_identity_sequence_id
        self.page_size = page_size


class CreateScopeInfo(Model):
    """
    :param admin_group_description:
    :type admin_group_description: str
    :param admin_group_name:
    :type admin_group_name: str
    :param creator_id:
    :type creator_id: str
    :param parent_scope_id:
    :type parent_scope_id: str
    :param scope_name:
    :type scope_name: str
    :param scope_type:
    :type scope_type: object
    """

    _attribute_map = {
        'admin_group_description': {'key': 'adminGroupDescription', 'type': 'str'},
        'admin_group_name': {'key': 'adminGroupName', 'type': 'str'},
        'creator_id': {'key': 'creatorId', 'type': 'str'},
        'parent_scope_id': {'key': 'parentScopeId', 'type': 'str'},
        'scope_name': {'key': 'scopeName', 'type': 'str'},
        'scope_type': {'key': 'scopeType', 'type': 'object'}
    }

    def __init__(self, admin_group_description=None, admin_group_name=None, creator_id=None, parent_scope_id=None, scope_name=None, scope_type=None):
        super(CreateScopeInfo, self).__init__()
        self.admin_group_description = admin_group_description
        self.admin_group_name = admin_group_name
        self.creator_id = creator_id
        self.parent_scope_id = parent_scope_id
        self.scope_name = scope_name
        self.scope_type = scope_type


class FrameworkIdentityInfo(Model):
    """
    :param display_name:
    :type display_name: str
    :param identifier:
    :type identifier: str
    :param identity_type:
    :type identity_type: object
    :param role:
    :type role: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'identifier': {'key': 'identifier', 'type': 'str'},
        'identity_type': {'key': 'identityType', 'type': 'object'},
        'role': {'key': 'role', 'type': 'str'}
    }

    def __init__(self, display_name=None, identifier=None, identity_type=None, role=None):
        super(FrameworkIdentityInfo, self).__init__()
        self.display_name = display_name
        self.identifier = identifier
        self.identity_type = identity_type
        self.role = role


class GroupMembership(Model):
    """
    :param active:
    :type active: bool
    :param descriptor:
    :type descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param id:
    :type id: str
    :param queried_id:
    :type queried_id: str
    """

    _attribute_map = {
        'active': {'key': 'active', 'type': 'bool'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'queried_id': {'key': 'queriedId', 'type': 'str'}
    }

    def __init__(self, active=None, descriptor=None, id=None, queried_id=None):
        super(GroupMembership, self).__init__()
        self.active = active
        self.descriptor = descriptor
        self.id = id
        self.queried_id = queried_id


class IdentityBase(Model):
    """
    Base Identity class to allow "trimmed" identity class in the GetConnectionData API Makes sure that on-the-wire representations of the derived classes are compatible with each other (e.g. Server responds with PublicIdentity object while client deserializes it as Identity object) Derived classes should not have additional [DataMember] properties

    :param custom_display_name: The custom display name for the identity (if any). Setting this property to an empty string will clear the existing custom display name. Setting this property to null will not affect the existing persisted value (since null values do not get sent over the wire or to the database)
    :type custom_display_name: str
    :param descriptor:
    :type descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param id: Identity Identifier. Also called Storage Key, or VSID
    :type id: str
    :param is_active: True if the identity has a membership in any Azure Devops group in the organization.
    :type is_active: bool
    :param is_container: True if the identity is a group.
    :type is_container: bool
    :param master_id:
    :type master_id: str
    :param member_ids: Id of the members of the identity (groups only).
    :type member_ids: list of str
    :param member_of:
    :type member_of: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    :param members:
    :type members: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    :param meta_type_id:
    :type meta_type_id: int
    :param properties:
    :type properties: :class:`object <azure.devops.v6_0.identities.models.object>`
    :param provider_display_name: The display name for the identity as specified by the source identity provider.
    :type provider_display_name: str
    :param resource_version:
    :type resource_version: int
    :param social_descriptor:
    :type social_descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param subject_descriptor: Subject descriptor of a Graph entity.
    :type subject_descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param unique_user_id:
    :type unique_user_id: int
    """

    _attribute_map = {
        'custom_display_name': {'key': 'customDisplayName', 'type': 'str'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'master_id': {'key': 'masterId', 'type': 'str'},
        'member_ids': {'key': 'memberIds', 'type': '[str]'},
        'member_of': {'key': 'memberOf', 'type': '[str]'},
        'members': {'key': 'members', 'type': '[str]'},
        'meta_type_id': {'key': 'metaTypeId', 'type': 'int'},
        'properties': {'key': 'properties', 'type': 'object'},
        'provider_display_name': {'key': 'providerDisplayName', 'type': 'str'},
        'resource_version': {'key': 'resourceVersion', 'type': 'int'},
        'social_descriptor': {'key': 'socialDescriptor', 'type': 'str'},
        'subject_descriptor': {'key': 'subjectDescriptor', 'type': 'str'},
        'unique_user_id': {'key': 'uniqueUserId', 'type': 'int'}
    }

    def __init__(self, custom_display_name=None, descriptor=None, id=None, is_active=None, is_container=None, master_id=None, member_ids=None, member_of=None, members=None, meta_type_id=None, properties=None, provider_display_name=None, resource_version=None, social_descriptor=None, subject_descriptor=None, unique_user_id=None):
        super(IdentityBase, self).__init__()
        self.custom_display_name = custom_display_name
        self.descriptor = descriptor
        self.id = id
        self.is_active = is_active
        self.is_container = is_container
        self.master_id = master_id
        self.member_ids = member_ids
        self.member_of = member_of
        self.members = members
        self.meta_type_id = meta_type_id
        self.properties = properties
        self.provider_display_name = provider_display_name
        self.resource_version = resource_version
        self.social_descriptor = social_descriptor
        self.subject_descriptor = subject_descriptor
        self.unique_user_id = unique_user_id


class IdentityBatchInfo(Model):
    """
    :param descriptors:
    :type descriptors: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    :param identity_ids:
    :type identity_ids: list of str
    :param include_restricted_visibility:
    :type include_restricted_visibility: bool
    :param property_names:
    :type property_names: list of str
    :param query_membership:
    :type query_membership: object
    :param social_descriptors:
    :type social_descriptors: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    :param subject_descriptors:
    :type subject_descriptors: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    """

    _attribute_map = {
        'descriptors': {'key': 'descriptors', 'type': '[str]'},
        'identity_ids': {'key': 'identityIds', 'type': '[str]'},
        'include_restricted_visibility': {'key': 'includeRestrictedVisibility', 'type': 'bool'},
        'property_names': {'key': 'propertyNames', 'type': '[str]'},
        'query_membership': {'key': 'queryMembership', 'type': 'object'},
        'social_descriptors': {'key': 'socialDescriptors', 'type': '[str]'},
        'subject_descriptors': {'key': 'subjectDescriptors', 'type': '[str]'}
    }

    def __init__(self, descriptors=None, identity_ids=None, include_restricted_visibility=None, property_names=None, query_membership=None, social_descriptors=None, subject_descriptors=None):
        super(IdentityBatchInfo, self).__init__()
        self.descriptors = descriptors
        self.identity_ids = identity_ids
        self.include_restricted_visibility = include_restricted_visibility
        self.property_names = property_names
        self.query_membership = query_membership
        self.social_descriptors = social_descriptors
        self.subject_descriptors = subject_descriptors


class IdentityRightsTransferData(Model):
    """
    :param user_principal_name_mappings:
    :type user_principal_name_mappings: dict
    """

    _attribute_map = {
        'user_principal_name_mappings': {'key': 'userPrincipalNameMappings', 'type': '{str}'}
    }

    def __init__(self, user_principal_name_mappings=None):
        super(IdentityRightsTransferData, self).__init__()
        self.user_principal_name_mappings = user_principal_name_mappings


class IdentityScope(Model):
    """
    :param administrators:
    :type administrators: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param id:
    :type id: str
    :param is_active:
    :type is_active: bool
    :param is_global:
    :type is_global: bool
    :param local_scope_id:
    :type local_scope_id: str
    :param name:
    :type name: str
    :param parent_id:
    :type parent_id: str
    :param scope_type:
    :type scope_type: object
    :param securing_host_id:
    :type securing_host_id: str
    :param subject_descriptor:
    :type subject_descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    """

    _attribute_map = {
        'administrators': {'key': 'administrators', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
        'is_global': {'key': 'isGlobal', 'type': 'bool'},
        'local_scope_id': {'key': 'localScopeId', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_id': {'key': 'parentId', 'type': 'str'},
        'scope_type': {'key': 'scopeType', 'type': 'object'},
        'securing_host_id': {'key': 'securingHostId', 'type': 'str'},
        'subject_descriptor': {'key': 'subjectDescriptor', 'type': 'str'}
    }

    def __init__(self, administrators=None, id=None, is_active=None, is_global=None, local_scope_id=None, name=None, parent_id=None, scope_type=None, securing_host_id=None, subject_descriptor=None):
        super(IdentityScope, self).__init__()
        self.administrators = administrators
        self.id = id
        self.is_active = is_active
        self.is_global = is_global
        self.local_scope_id = local_scope_id
        self.name = name
        self.parent_id = parent_id
        self.scope_type = scope_type
        self.securing_host_id = securing_host_id
        self.subject_descriptor = subject_descriptor


class IdentitySelf(Model):
    """
    Identity information.

    :param account_name: The UserPrincipalName (UPN) of the account. This value comes from the source provider.
    :type account_name: str
    :param display_name: The display name. For AAD accounts with multiple tenants this is the display name of the profile in the home tenant.
    :type display_name: str
    :param domain: This represents the name of the container of origin. For AAD accounts this is the tenantID of the home tenant. For MSA accounts this is the string "Windows Live ID".
    :type domain: str
    :param id: This is the VSID of the home tenant profile. If the profile is signed into the home tenant or if the profile has no tenants then this Id is the same as the Id returned by the profile/profiles/me endpoint. Going forward it is recommended that you use the combined values of Origin, OriginId and Domain to uniquely identify a user rather than this Id.
    :type id: str
    :param origin: The type of source provider for the origin identifier. For MSA accounts this is "msa". For AAD accounts this is "aad".
    :type origin: str
    :param origin_id: The unique identifier from the system of origin. If there are multiple tenants this is the unique identifier of the account in the home tenant. (For MSA this is the PUID in hex notation, for AAD this is the object id.)
    :type origin_id: str
    :param tenants: For AAD accounts this is all of the tenants that this account is a member of.
    :type tenants: list of :class:`TenantInfo <azure.devops.v6_0.identities.models.TenantInfo>`
    """

    _attribute_map = {
        'account_name': {'key': 'accountName', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'domain': {'key': 'domain', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'origin': {'key': 'origin', 'type': 'str'},
        'origin_id': {'key': 'originId', 'type': 'str'},
        'tenants': {'key': 'tenants', 'type': '[TenantInfo]'}
    }

    def __init__(self, account_name=None, display_name=None, domain=None, id=None, origin=None, origin_id=None, tenants=None):
        super(IdentitySelf, self).__init__()
        self.account_name = account_name
        self.display_name = display_name
        self.domain = domain
        self.id = id
        self.origin = origin
        self.origin_id = origin_id
        self.tenants = tenants


class IdentitySnapshot(Model):
    """
    :param groups:
    :type groups: list of :class:`Identity <azure.devops.v6_0.identities.models.Identity>`
    :param identity_ids:
    :type identity_ids: list of str
    :param memberships:
    :type memberships: list of :class:`GroupMembership <azure.devops.v6_0.identities.models.GroupMembership>`
    :param scope_id:
    :type scope_id: str
    :param scopes:
    :type scopes: list of :class:`IdentityScope <azure.devops.v6_0.identities.models.IdentityScope>`
    """

    _attribute_map = {
        'groups': {'key': 'groups', 'type': '[Identity]'},
        'identity_ids': {'key': 'identityIds', 'type': '[str]'},
        'memberships': {'key': 'memberships', 'type': '[GroupMembership]'},
        'scope_id': {'key': 'scopeId', 'type': 'str'},
        'scopes': {'key': 'scopes', 'type': '[IdentityScope]'}
    }

    def __init__(self, groups=None, identity_ids=None, memberships=None, scope_id=None, scopes=None):
        super(IdentitySnapshot, self).__init__()
        self.groups = groups
        self.identity_ids = identity_ids
        self.memberships = memberships
        self.scope_id = scope_id
        self.scopes = scopes


class IdentityUpdateData(Model):
    """
    :param id:
    :type id: str
    :param index:
    :type index: int
    :param updated:
    :type updated: bool
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'index': {'key': 'index', 'type': 'int'},
        'updated': {'key': 'updated', 'type': 'bool'}
    }

    def __init__(self, id=None, index=None, updated=None):
        super(IdentityUpdateData, self).__init__()
        self.id = id
        self.index = index
        self.updated = updated


class JsonPatchOperation(Model):
    """
    The JSON model for a JSON Patch operation

    :param from_: The path to copy from for the Move/Copy operation.
    :type from_: str
    :param op: The patch operation
    :type op: object
    :param path: The path for the operation. In the case of an array, a zero based index can be used to specify the position in the array (e.g. /biscuits/0/name). The "-" character can be used instead of an index to insert at the end of the array (e.g. /biscuits/-).
    :type path: str
    :param value: The value for the operation. This is either a primitive or a JToken.
    :type value: object
    """

    _attribute_map = {
        'from_': {'key': 'from', 'type': 'str'},
        'op': {'key': 'op', 'type': 'object'},
        'path': {'key': 'path', 'type': 'str'},
        'value': {'key': 'value', 'type': 'object'}
    }

    def __init__(self, from_=None, op=None, path=None, value=None):
        super(JsonPatchOperation, self).__init__()
        self.from_ = from_
        self.op = op
        self.path = path
        self.value = value


class JsonWebToken(Model):
    """
    """

    _attribute_map = {
    }

    def __init__(self):
        super(JsonWebToken, self).__init__()


class PagedIdentities(Model):
    """
    :param continuation_token:
    :type continuation_token: list of str
    :param identities:
    :type identities: list of :class:`Identity <azure.devops.v6_0.identities.models.Identity>`
    """

    _attribute_map = {
        'continuation_token': {'key': 'continuationToken', 'type': '[str]'},
        'identities': {'key': 'identities', 'type': '[Identity]'}
    }

    def __init__(self, continuation_token=None, identities=None):
        super(PagedIdentities, self).__init__()
        self.continuation_token = continuation_token
        self.identities = identities


class RefreshTokenGrant(AuthorizationGrant):
    """
    :param grant_type:
    :type grant_type: object
    :param jwt:
    :type jwt: :class:`JsonWebToken <azure.devops.v6_0.microsoft._visual_studio._services._web_api.models.JsonWebToken>`
    """

    _attribute_map = {
        'grant_type': {'key': 'grantType', 'type': 'object'},
        'jwt': {'key': 'jwt', 'type': 'JsonWebToken'}
    }

    def __init__(self, grant_type=None, jwt=None):
        super(RefreshTokenGrant, self).__init__(grant_type=grant_type)
        self.jwt = jwt


class SwapIdentityInfo(Model):
    """
    :param id1:
    :type id1: str
    :param id2:
    :type id2: str
    """

    _attribute_map = {
        'id1': {'key': 'id1', 'type': 'str'},
        'id2': {'key': 'id2', 'type': 'str'}
    }

    def __init__(self, id1=None, id2=None):
        super(SwapIdentityInfo, self).__init__()
        self.id1 = id1
        self.id2 = id2


class TenantInfo(Model):
    """
    :param home_tenant:
    :type home_tenant: bool
    :param tenant_id:
    :type tenant_id: str
    :param tenant_name:
    :type tenant_name: str
    :param verified_domains:
    :type verified_domains: list of str
    """

    _attribute_map = {
        'home_tenant': {'key': 'homeTenant', 'type': 'bool'},
        'tenant_id': {'key': 'tenantId', 'type': 'str'},
        'tenant_name': {'key': 'tenantName', 'type': 'str'},
        'verified_domains': {'key': 'verifiedDomains', 'type': '[str]'}
    }

    def __init__(self, home_tenant=None, tenant_id=None, tenant_name=None, verified_domains=None):
        super(TenantInfo, self).__init__()
        self.home_tenant = home_tenant
        self.tenant_id = tenant_id
        self.tenant_name = tenant_name
        self.verified_domains = verified_domains


class Identity(IdentityBase):
    """
    :param custom_display_name: The custom display name for the identity (if any). Setting this property to an empty string will clear the existing custom display name. Setting this property to null will not affect the existing persisted value (since null values do not get sent over the wire or to the database)
    :type custom_display_name: str
    :param descriptor:
    :type descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param id: Identity Identifier. Also called Storage Key, or VSID
    :type id: str
    :param is_active: True if the identity has a membership in any Azure Devops group in the organization.
    :type is_active: bool
    :param is_container: True if the identity is a group.
    :type is_container: bool
    :param master_id:
    :type master_id: str
    :param member_ids: Id of the members of the identity (groups only).
    :type member_ids: list of str
    :param member_of:
    :type member_of: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    :param members:
    :type members: list of :class:`str <azure.devops.v6_0.identities.models.str>`
    :param meta_type_id:
    :type meta_type_id: int
    :param properties:
    :type properties: :class:`object <azure.devops.v6_0.identities.models.object>`
    :param provider_display_name: The display name for the identity as specified by the source identity provider.
    :type provider_display_name: str
    :param resource_version:
    :type resource_version: int
    :param social_descriptor:
    :type social_descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param subject_descriptor: Subject descriptor of a Graph entity.
    :type subject_descriptor: :class:`str <azure.devops.v6_0.identities.models.str>`
    :param unique_user_id:
    :type unique_user_id: int
    """

    _attribute_map = {
        'custom_display_name': {'key': 'customDisplayName', 'type': 'str'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'master_id': {'key': 'masterId', 'type': 'str'},
        'member_ids': {'key': 'memberIds', 'type': '[str]'},
        'member_of': {'key': 'memberOf', 'type': '[str]'},
        'members': {'key': 'members', 'type': '[str]'},
        'meta_type_id': {'key': 'metaTypeId', 'type': 'int'},
        'properties': {'key': 'properties', 'type': 'object'},
        'provider_display_name': {'key': 'providerDisplayName', 'type': 'str'},
        'resource_version': {'key': 'resourceVersion', 'type': 'int'},
        'social_descriptor': {'key': 'socialDescriptor', 'type': 'str'},
        'subject_descriptor': {'key': 'subjectDescriptor', 'type': 'str'},
        'unique_user_id': {'key': 'uniqueUserId', 'type': 'int'},
    }

    def __init__(self, custom_display_name=None, descriptor=None, id=None, is_active=None, is_container=None, master_id=None, member_ids=None, member_of=None, members=None, meta_type_id=None, properties=None, provider_display_name=None, resource_version=None, social_descriptor=None, subject_descriptor=None, unique_user_id=None):
        super(Identity, self).__init__(custom_display_name=custom_display_name, descriptor=descriptor, id=id, is_active=is_active, is_container=is_container, master_id=master_id, member_ids=member_ids, member_of=member_of, members=members, meta_type_id=meta_type_id, properties=properties, provider_display_name=provider_display_name, resource_version=resource_version, social_descriptor=social_descriptor, subject_descriptor=subject_descriptor, unique_user_id=unique_user_id)


__all__ = [
    'AccessTokenResult',
    'AuthorizationGrant',
    'ChangedIdentities',
    'ChangedIdentitiesContext',
    'CreateScopeInfo',
    'FrameworkIdentityInfo',
    'GroupMembership',
    'IdentityBase',
    'IdentityBatchInfo',
    'IdentityRightsTransferData',
    'IdentityScope',
    'IdentitySelf',
    'IdentitySnapshot',
    'IdentityUpdateData',
    'JsonPatchOperation',
    'JsonWebToken',
    'PagedIdentities',
    'RefreshTokenGrant',
    'SwapIdentityInfo',
    'TenantInfo',
    'Identity',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from webbrowser import open_new

from knack.log import get_logger
from azext_devops.dev.common.services import (get_release_client, resolve_instance_and_project)
from azext_devops.dev.common.uuid import is_uuid

logger = get_logger(__name__)


def release_definition_list(name=None, top=None, organization=None, project=None,
                            artifact_type=None, artifact_source_id=None, detect=None):
    """List release definitions.
    :param name: Limit results to definitions with this name or contains this name. Example: "FabCI"
    :type name: str
    :param top: Maximum number of definitions to list.
    :type top: int
    :param artifact_type: Release definitions with given artifactType will be returned.
    :type artifact_type: str
    :param artifact_source_id: Limit results to definitions associated with this artifact_source_id.
    e.g. For build it would be {projectGuid}:{BuildDefinitionId}, for Jenkins it would be
    {JenkinsConnectionId}:{JenkinsDefinitionId}, for TfsOnPrem it would be
    {TfsOnPremConnectionId}:{ProjectName}:{TfsOnPremDefinitionId}.
    For third-party artifacts e.g. TeamCity, BitBucket you may refer 'uniqueSourceIdentifier'
    inside vss-extension.json at https://github.com/Microsoft/vsts-rm-extensions/blob/master/Extensions.
    :type artifact_source_id: str
    :rtype: [ReleaseDefinitionReference]
    """
    organization, project = resolve_instance_and_project(
        detect=detect, organization=organization, project=project)
    client = get_release_client(organization)
    query_order = 'nameAscending'
    definition_references = client.get_release_definitions(
        project=project, search_text=name, artifact_source_id=artifact_source_id, artifact_type=artifact_type,
        top=top, query_order=query_order)
    return definition_references


def release_definition_show(id=None, name=None, open=False, organization=None, project=None,  # pylint: disable=redefined-builtin
                            detect=None):
    """Get the details of a release definition.
    :param id: ID of the definition.
    :type id: int
    :param name: Name of the definition. Ignored if --id is supplied.
    :type name: str
    :param open: Open the definition summary page in your web browser.
    :type open: bool
    :rtype: ReleaseDefinitionReference
    """
    organization, project = resolve_instance_and_project(
        detect=detect, organization=organization, project=project)
    client = get_release_client(organization)
    if id is None:
        if name is not None:
            id = get_definition_id_from_name(name, client, project)
        else:
            raise ValueError("Either the --id argument or the --name argument must be supplied for this command.")
    release_definition = client.get_release_definition(definition_id=id, project=project)
    if open:
        _open_definition(release_definition)
    return release_definition


def _open_definition(definition):
    """Opens the release definition in the default browser.
    :param :class:`<ReleaseDefinitionReference> <v5_0.release.models.ReleaseDefinitionReference>` definition:
    """
    url = _get_release_definition_web_url(definition)
    if url is not None and url:
        logger.debug('Opening web page: %s', url)
        open_new(url=url)


def get_definition_id_from_name(name, client, project):
    definition_references = client.get_release_definitions(
        project=project, search_text=name, is_exact_name_match='true')
    if len(definition_references) == 1:
        return definition_references[0].id
    if len(definition_references) > 1:
        if is_uuid(project):
            project = definition_references[0].project.name
        message = 'Multiple definitions were found matching name "{name}" in project "{project}".  Try '\
                  + 'supplying the definition ID.'
        raise ValueError(message.format(name=name, project=project))

    raise ValueError('There were no release definitions matching name "{name}" in project "{project}".'
                     .format(name=name, project=project))


def _get_release_definition_web_url(definition):
    links = definition._links   # pylint: disable=protected-access
    if links is not None and links:
        properties = links.additional_properties
        if properties is not None and properties:
            web_url = properties.get('web')
            if web_url is not None and web_url:
                return web_url.get('href')
    return None



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from knack.arguments import enum_choice_list
from azure.cli.core.commands.parameters import get_enum_type, get_three_state_flag
from azext_devops.dev.common.utils import FILE_ENCODING_TYPES


# CUSTOM CHOICE LISTS
_SOURCE_CONTROL_VALUES = ['git', 'tfvc']
_WIKI_TYPE_VALUES = ['projectwiki', 'codewiki']
_SCOPE_VALUES = ['project', 'organization']
_PROJECT_VISIBILITY_VALUES = ['private', 'public']
_STATE_VALUES = ['invalid', 'unchanged', 'all', 'new', 'wellformed', 'deleting', 'createpending']
_PROJECT_GET_STATE_VALUE_FILTER = ['all', 'createPending', 'deleted', 'deleting', 'new', 'unchanged', 'wellFormed']

_HTTP_METHOD_VALUES = ['GET', 'POST', 'PATCH', 'DELETE', 'OPTIONS', 'PUT', 'HEAD']

_LICENSE_TYPES = ['advanced', 'earlyAdopter', 'express', 'professional', 'stakeholder']
_RELATIONSHIP_TYPES = ['members', 'memberof']
_FILE_ENCODING_TYPE_VALUES = FILE_ENCODING_TYPES


def load_global_args(context):
    context.argument('organization', options_list=('--organization', '--org'),
                     help='Azure DevOps organization URL. You can configure the default organization using '
                     'az devops configure -d organization=ORG_URL. Required if not configured as '
                     'default or picked up via git config. Example: https://dev.azure.com/MyOrganizationName/')
    context.argument('detect', arg_type=get_three_state_flag(),
                     help='Automatically detect organization.')
    context.argument('project', options_list=('--project', '-p'),
                     help='Name or ID of the project. You can configure the default project using '
                     'az devops configure -d project=NAME_OR_ID. Required if not configured as '
                     'default or picked up via git config.')


# pylint: disable=too-many-statements
def load_team_arguments(self, _):
    with self.argument_context('devops login') as context:
        context.argument('organization',
                         help='Azure DevOps organization URL. Example: https://dev.azure.com/MyOrganizationName')

    with self.argument_context('devops logout') as context:
        context.argument('organization',
                         help='Azure DevOps organization URL. Example: https://dev.azure.com/MyOrganizationName/. '
                         'If no organization is specified, all organizations will be logged out.')

    with self.argument_context('devops configure') as context:
        context.argument('defaults', options_list=('--defaults', '-d'), nargs='*')
        context.argument('use_git_aliases', arg_type=get_three_state_flag())
        context.argument('list_config', options_list=('--list', '-l'))

    with self.argument_context('devops') as context:
        context.argument('repository', options_list=('--repository', '-r'))

    with self.argument_context('devops project') as context:
        context.argument('process', options_list=('--process', '-p'))
        context.argument('source_control', options_list=('--source-control', '-s'),
                         **enum_choice_list(_SOURCE_CONTROL_VALUES))
        context.argument('description', options_list=('--description', '-d'))
        context.argument('state', **enum_choice_list(_STATE_VALUES))
        context.argument('visibility', **enum_choice_list(_PROJECT_VISIBILITY_VALUES))

    with self.argument_context('devops project delete') as context:
        context.argument('yes', options_list=['--yes', '-y'], action='store_true',
                         help='Do not prompt for confirmation.')

    with self.argument_context('devops project list') as context:
        context.argument('state_filter', arg_type=get_enum_type(_PROJECT_GET_STATE_VALUE_FILTER),
                         help='State filter.')
        context.argument('continuation_token',
                         help='Continuation token. '
                         'This can be retrived from previous run of this command if more results are present.')
        context.argument('get_default_team_image_url', arg_type=get_three_state_flag(),
                         help='Whether to get default team image url or not.')

    with self.argument_context('devops service-endpoint create') as context:
        context.argument('encoding',
                         help='Encoding of the input file.',
                         **enum_choice_list(_FILE_ENCODING_TYPE_VALUES))

    with self.argument_context('devops service-endpoint update') as context:
        context.argument('enable_for_all',
                         help='Allow all pipelines to access this service endpoint.',
                         arg_type=get_three_state_flag())

    with self.argument_context('devops invoke') as context:
        context.argument('route_parameters', nargs='*',
                         help='Specifies the list of route parameters')
        context.argument('query_parameters', nargs='*',
                         help='Specifies the list of query parameters')
        context.argument('http_method', arg_type=get_enum_type(_HTTP_METHOD_VALUES),
                         help='Specifies the method used for the request.')
        context.argument('media_type',
                         help='Specifies the content type of the request.')
        context.argument('accept_media_type',
                         help='Specifies the content type of the response.')
        context.argument('in_file',
                         help='Path and file name to the file that contains the contents of the request.')
        context.argument('encoding',
                         help='Encoding of the input file. Used in conjunction with --in-file.',
                         **enum_choice_list(_FILE_ENCODING_TYPE_VALUES))
        context.argument('out_file',
                         help='Path and file name to the file  for which this function saves the response body.')
        context.argument('area',
                         help='The area to find the resource.')
        context.argument('resource',
                         help='The name of the resource to operate on.')
        context.argument('api_version',
                         help='The version of the API to target')

    with self.argument_context('devops user') as context:
        context.argument('license_type', arg_type=get_enum_type(_LICENSE_TYPES))
    with self.argument_context('devops user add') as context:
        context.argument('send_email_invite', arg_type=get_three_state_flag(),
                         help='Whether to send email invite for new user or not.')

    with self.argument_context('devops security group create') as context:
        context.argument('project',
                         help='Name or ID of the project in which Azure DevOps group should be created.')
        context.argument('scope', **enum_choice_list(_SCOPE_VALUES))

    with self.argument_context('devops security group list') as context:
        context.argument('project',
                         help='List groups for a particular project')
        context.argument('scope', **enum_choice_list(_SCOPE_VALUES))

    with self.argument_context('devops security group membership') as context:
        context.argument('relationship', arg_type=get_enum_type(_RELATIONSHIP_TYPES),
                         help='Get member of/members for this group.')

    with self.argument_context('devops security permission') as context:
        context.argument('namespace_id', options_list=('--namespace-id', '--id'),
                         help='ID of security namespace')
        context.argument('token',
                         help='Security token.')
        context.argument('subject',
                         help='User Email ID or Group descriptor')

    with self.argument_context('devops security permission update') as context:
        context.argument('merge', arg_type=get_three_state_flag(),
                         help='If set, the existing ACE has its allow and deny merged with \
                         the incoming ACE\'s allow and deny. If unset, the existing ACE is displaced.')
        context.argument('allow_bit', type=int,
                         help='Allow bit or addition of bits. Required if --deny-bit is missing.')
        context.argument('deny_bit', type=int,
                         help='Deny bit or addition of bits. Required if --allow-bit is missing.')

    with self.argument_context('devops security permission reset') as context:
        context.argument('permission_bit', type=int,
                         help='Permission bit or addition of permission bits which needs to be reset\
                         for given user/group and token.')

    with self.argument_context('devops extension') as context:
        context.argument('include_built_in', arg_type=get_three_state_flag(),
                         help='Include built in extensions.')
        context.argument('include_disabled', arg_type=get_three_state_flag(),
                         help='Include disabled extensions.')
        context.argument('publisher_id', help='Publisher Id. This will map to publisher-name \
                          in the az devops extension search output. ')
        context.argument('extension_id', help='Extension Id. This will map to extension-name \
                          in the az devops extension search output.')
        context.argument('search_query', options_list=('--search-query', '-q'), help='Search term')

    with self.argument_context('devops') as context:
        load_global_args(context)

    with self.argument_context('repos') as context:
        load_global_args(context)

    with self.argument_context('artifacts') as context:
        load_global_args(context)

    with self.argument_context('boards') as context:
        load_global_args(context)

    with self.argument_context('pipelines') as context:
        load_global_args(context)

    with self.argument_context('devops wiki') as context:
        context.argument('wiki_type', options_list=('--wiki-type', '--type'), **enum_choice_list(_WIKI_TYPE_VALUES))
        context.argument('version', options_list=('--version', '-v'))
        context.argument('encoding', **enum_choice_list(_FILE_ENCODING_TYPE_VALUES))

    with self.argument_context('devops wiki list') as context:
        context.argument('scope', **enum_choice_list(_SCOPE_VALUES))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest

from azure_devtools.scenario_tests import AllowLargeResponse
from .utilities.helper import DevopsScenarioTest, disable_telemetry, set_authentication, get_test_org_from_env_variable

DEVOPS_CLI_TEST_ORGANIZATION = get_test_org_from_env_variable() or 'Https://dev.azure.com/azuredevopsclitest'

class PipelinesBuildDefinitionTests(DevopsScenarioTest):
    @AllowLargeResponse(size_kb=3072)
    @disable_telemetry
    @set_authentication
    def test_build_definition_listShow(self):
        self.cmd('az devops configure --defaults organization=' + DEVOPS_CLI_TEST_ORGANIZATION + ' project=buildtests')

        build_definition_name = 'BuildTests Definition1'

        #list build definition
        list_build_definition_command = 'az pipelines build definition list --detect false --output json'
        list_build_definition_output = self.cmd(list_build_definition_command).get_output_in_json()
        assert len(list_build_definition_output) > 0
        for build_definition in list_build_definition_output:
            if(build_definition["name"] == build_definition_name):
                build_definition_id = build_definition["id"]
            assert build_definition["id"] > 0


        #show build definition
        show_build_definition_command = 'az pipelines build definition show --id ' + str(build_definition_id) + ' --detect false --output json'
        show_build_definition_output = self.cmd(show_build_definition_command).get_output_in_json()
        assert len(show_build_definition_output) > 0
        assert show_build_definition_output["name"] == build_definition_name
   


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function
from collections import OrderedDict
from knack.util import CLIError
from azext_devops.dev.common.format import trim_for_display, date_time_to_only_date


def transform_extension_search_results_table_output(result):
    table_output = []
    for item in result:
        table_output.append(_transform_extension_search_result_row(item))
    return table_output


def _transform_extension_search_result_row(row):
    table_row = OrderedDict()
    table_row['Publisher Name'] = row['publisher']['publisherName']
    table_row['Extension Name'] = row['extensionName']
    table_row['Name'] = row['displayName']

    return table_row


def transform_extension_table_output(result):
    table_output = [_transform_extension_row(result)]
    return table_output


def transform_extensions_table_output(result):
    table_output = []
    for item in sorted(result, key=_get_extension_key):
        table_output.append(_transform_extension_row(item))
    return table_output


def _transform_extension_row(row):
    table_row = OrderedDict()
    table_row['Publisher Id'] = trim_for_display(row['publisherId'], 10)
    table_row['Extension Id'] = trim_for_display(row['extensionId'], 20)
    table_row['Name'] = trim_for_display(row['extensionName'], 20)
    table_row['Version '] = trim_for_display(row['version'], 20)
    table_row['Last Updated '] = date_time_to_only_date(row['lastPublished'])
    table_row['States'] = trim_for_display(row['installState']['flags'], 20)
    table_row['Flags'] = trim_for_display(row['flags'], 20)

    return table_row


def transform_projects_table_output(result):
    table_output = []
    for item in sorted(result['value'], key=_get_project_key):
        table_output.append(_transform_project_row(item))
    return table_output


def transform_project_table_output(result):
    table_output = [_transform_project_row(result)]
    return table_output


def _transform_project_row(row):
    from .project import (PROCESS_TEMPLATE_CAPABILITY_NAME,
                          VERSION_CONTROL_CAPABILITY_NAME,
                          VERSION_CONTROL_CAPABILITY_ATTRIBUTE_NAME)
    table_row = OrderedDict()
    table_row['ID'] = row['id']
    table_row['Name'] = row['name']
    table_row['Visibility'] = row['visibility'].capitalize()

    if 'capabilities' in row:
        capabilities = row['capabilities']
        if PROCESS_TEMPLATE_CAPABILITY_NAME in capabilities:
            process_capabilities = capabilities[PROCESS_TEMPLATE_CAPABILITY_NAME]
            if 'templateName' in process_capabilities:
                table_row['Process'] = process_capabilities['templateName']
        if VERSION_CONTROL_CAPABILITY_NAME in capabilities:
            version_capabilities = capabilities[VERSION_CONTROL_CAPABILITY_NAME]
            if VERSION_CONTROL_CAPABILITY_ATTRIBUTE_NAME in version_capabilities:
                table_row['Source Control'] = version_capabilities[VERSION_CONTROL_CAPABILITY_ATTRIBUTE_NAME]

    return table_row


def transform_service_endpoints_table_output(result):
    table_output = []
    for item in sorted(result, key=_get_service_endpoint_key):
        table_output.append(_transform_service_endpoint_row(item))
    return table_output


def _transform_service_endpoint_row(row):
    table_row = OrderedDict()
    table_row['ID'] = row['id']
    table_row['Name'] = row['name']
    table_row['Type'] = row['type']
    table_row['Is Ready'] = row['isReady']
    table_row['Created By'] = row['createdBy']['displayName']

    return table_row


def transform_authorized_service_endpoint_table_output(result):
    table_output = []
    table_output.append(_transform_authorized_service_endpoint_row(result))
    return table_output


def _transform_authorized_service_endpoint_row(row):
    table_row = OrderedDict()
    table_row['ID'] = row['id']
    table_row['Allow Pipelines'] = row['authorized']
    table_row['Name'] = row['name']
    table_row['Type'] = row['type']
    table_row['Is Ready'] = row['isReady']
    table_row['Created By'] = row['createdBy']['displayName']

    return table_row


def transform_groups_table_output(result):
    table_output = []
    if result['continuationToken'] is not None:
        print('Showing only 500 groups. ' +
              'To list next set of groups use this token as --continuation-token argument and run the command again.' +
              ' TOKEN:', result['continuationToken'])
    for item in result['graphGroups']:
        table_output.append(_transform_group_row(item))
    return table_output


def transform_group_table_output(result):
    table_output = [_transform_group_show_table_output(result)]
    return table_output


def _transform_group_show_table_output(row):
    table_row = OrderedDict()
    table_row['Name'] = row['principalName']
    table_row['Description'] = row['description']
    return table_row


def _transform_group_row(row):
    table_row = OrderedDict()
    table_row['Name'] = row['principalName']
    table_row['Descriptor'] = row['descriptor']
    return table_row


def transform_memberships_table_output(result):
    table_output = []
    for item in result:
        table_output.append(_transform_membership_row(result[item]))
    return table_output


def transform_membership_table_output(result):
    table_output = []
    for item in result:
        table_row = OrderedDict()
        row = result[item]
        if row['subjectKind'] == 'user':
            table_row['Name'] = row['displayName']
        else:
            table_row['Name'] = row['principalName']
        table_row['Type'] = row['subjectKind']
        table_row['Email'] = row['mailAddress']
        table_output.append(table_row)
    return table_output


def _transform_membership_row(row):
    table_row = OrderedDict()
    if row['subjectKind'] == 'user':
        table_row['Name'] = row['displayName']
    else:
        table_row['Name'] = row['principalName']
    table_row['Type'] = row['subjectKind']
    table_row['Email'] = row['mailAddress']
    table_row['Descriptor'] = row['descriptor']
    return table_row


def transform_namespaces_table_output(result):
    table_output = []
    for item in result:
        table_output.append(_transform_namespace_row(item))
    return table_output


def _transform_namespace_row(row):
    table_row = OrderedDict()
    table_row['Id'] = row['namespaceId']
    table_row['Name'] = row['name']
    return table_row


def transform_namespace_table_output(result):
    table_output = []
    for item in result[0]['actions']:
        table_output.append(_transform_namespace_details_row(item))
    return table_output


def _transform_namespace_details_row(row):
    table_row = OrderedDict()
    table_row['Name'] = row['name']
    table_row['Permission Description'] = row['displayName']
    table_row['Permission Bit'] = row['bit']
    return table_row


def transform_acl_output(result):
    table_output = []
    for item in result:
        table_output.append(_transform_acl_details_row(item))
    return table_output


def _transform_acl_details_row(row):
    if len(row['acesDictionary']) > 1:
        raise CLIError('More than one entry found in Aces dictionary for this user/group.')
    table_row = OrderedDict()
    table_row['token'] = row['token']
    ace = list(row['acesDictionary'].values())[0]
    if row['includeExtendedInfo']:
        if ace['extendedInfo']['effectiveAllow'] is not None:
            table_row['Effective Allow'] = ace['extendedInfo']['effectiveAllow']
        else:
            table_row['Effective Allow'] = 0
        if ace['extendedInfo']['effectiveDeny'] is not None:
            table_row['Effective Deny'] = ace['extendedInfo']['effectiveDeny']
        else:
            table_row['Effective Deny'] = 0
    return table_row


def transform_resolve_permission_bits(result):
    table_output = []
    ace_entry = list(result[0]['acesDictionary'].values())[0]
    permissions = ace_entry['resolvedPermissions']
    for permission in permissions:
        table_output.append(_transform_resolve_bits_row(permission))
    return table_output


def _transform_resolve_bits_row(row):
    table_row = OrderedDict()
    table_row['Name'] = row['name']
    table_row['Bit'] = row['bit']
    table_row['Permission Description'] = row['displayName']
    table_row['Permission Value'] = row['effectivePermission']
    return table_row


def transform_teams_table_output(result):
    table_output = []
    for item in sorted(result, key=_get_team_key):
        table_output.append(_transform_team_row(item))
    return table_output


def transform_team_table_output(result):
    table_output = [_transform_team_row(result)]
    return table_output


def _transform_team_row(row):
    table_row = OrderedDict()
    table_row['ID'] = row['id']
    table_row['Name'] = row['name']
    table_row['Description'] = row['description']

    return table_row


def transform_wikis_table_output(result):
    table_output = []
    for item in sorted(result, key=_get_wiki_key):
        table_output.append(_transform_wiki_row(item))
    return table_output


def transform_wiki_table_output(result):
    table_output = [_transform_wiki_row(result)]
    return table_output


def _transform_wiki_row(row):
    table_row = OrderedDict()
    table_row['ID'] = row['id']
    table_row['Name'] = row['name']
    table_row['Type'] = row['type']
    return table_row


def transform_wiki_page_table_output(result):
    table_output = [_transform_wiki_page_row(result)]
    return table_output


def _transform_wiki_page_row(row):
    table_row = OrderedDict()
    table_row['ETag'] = row['eTag']
    table_row['Page Path'] = '\'{}\''.format(row['page']['path'])
    table_row['Is Parent'] = row['page']['isParentPage']
    table_row['order'] = row['page']['order']
    return table_row


def transform_team_members_table_output(result):
    table_output = []
    for item in sorted(result, key=_get_member_key):
        table_output.append(_transform_team_member_row(item))
    return table_output


def _transform_team_member_row(row):
    table_row = OrderedDict()
    table_row['ID'] = row['identity']['id']
    table_row['Name'] = row['identity']['displayName']
    table_row['Email'] = row['identity']['uniqueName']

    return table_row


def transform_users_table_output(result):
    members = result['members']
    table_output = []
    for item in members:
        table_output.append(_transform_user_row(item))
    return table_output


def transform_user_table_output(result):
    table_output = [_transform_user_row(result)]
    return table_output


def _transform_user_row(row):
    table_row = OrderedDict()
    table_row['ID'] = row['id']
    table_row['Display Name'] = row['user']['displayName']
    table_row['Email'] = row['user']['mailAddress']
    table_row['License Type'] = row['accessLevel']['accountLicenseType']
    table_row['Access Level'] = row['accessLevel']['licenseDisplayName']
    table_row['Status'] = row['accessLevel']['status']
    return table_row


def _get_extension_key(extension):
    return extension['extensionName'].lower()


def _get_permission_key(permission_row):
    return permission_row['displayName'].lower()


def _get_service_endpoint_key(service_endpoint_row):
    return service_endpoint_row['name'].lower()


def _get_project_key(project_row):
    return project_row['name'].lower()


def _get_team_key(team_row):
    return team_row['name'].lower()


def _get_wiki_key(wiki_row):
    return wiki_row['name'].lower()


def _get_member_key(member_row):
    return member_row['identity']['uniqueName'].lower()
